@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Command line utility to manage an index
- *                                                      Copyright (C) 2003-2004 Mikio Hirabayashi
+ *                                                      Copyright (C) 2003-2005 Mikio Hirabayashi
  * This file is part of Estraier, a personal full-text search system.
  * Estraier is free software; you can redistribute it and/or modify it under the terms of the GNU
  * General Public License as published by the Free Software Foundation; either version 2 of the
@@ -25,12 +25,12 @@
 #define DLFILTERPRE    '@'               /* prefix of a dynamic linking function */
 #define RCINDEXBNUM    163819            /* ibnum tuning of rich mode */
 #define RCINDEXDNUM    11                /* idnum tuning of rich mode */
-#define RCCACHEBNUM    131071            /* cbnum tuning of rich mode */
-#define RCCACHESIZ     8388608           /* csiz tuning of rich mode */
+#define RCCACHEBNUM    262139            /* cbnum tuning of rich mode */
+#define RCCACHESIZ     10485760          /* csiz tuning of rich mode */
 #define PLINDEXBNUM    163819            /* ibnum tuning of plute mode */
 #define PLINDEXDNUM    17                /* idnum tuning of plute mode */
-#define PLCACHEBNUM    262144            /* cbnum tuning of plute mode */
-#define PLCACHESIZ     16777216          /* csiz tuning of plute mode */
+#define PLCACHEBNUM    524288            /* cbnum tuning of plute mode */
+#define PLCACHESIZ     20971520          /* csiz tuning of plute mode */
 #define NICEINC        10                /* increment value for nice */
 #define RLXSLEEPSEC    1                 /* number of sleeping seconds for relax */
 #define RLXPTMAX       1024              /* max point for relax */
@@ -53,6 +53,7 @@ int mnude = FALSE;                       /* whether attributes of the inner to b
 CBLIST *xsuflist = NULL;                 /* list of suffixes for the external filter */
 const char *xtype = NULL;                /* media type for outer filter */
 const char *xcmd = NULL;                 /* command of outer filter */
+CBMAP *typemap = NULL;                   /* map of media types and outer filter */
 int xtext = FALSE;                       /* whether to use the outer filter of plain text */
 int xmime = FALSE;                       /* whether to use the outer filter of MIME */
 int izero = FALSE;                       /* whether to ignore empty documents */
@@ -173,7 +174,7 @@ void usage(void){
   fprintf(stderr, "\n");
   fprintf(stderr, "usage:\n");
   fprintf(stderr, "  %s register [-list file] [-force] [-relax] [-wmax num]"
-          " [-tsuf sufs] [-hsuf sufs] [-msuf sufs] [-mn] [-xsuf sufs type cmd]"
+          " [-tsuf sufs] [-hsuf sufs] [-msuf sufs] [-mn] [-xsuf sufs type cmd] [-xtype type cmd]"
           " [-xt] [-xm] [-iz] [-ipre pres] [-isiz size] [-enc code] [-pt code] [-ft code]"
           " [-tattr attrs] [-rich] [-plute] name [dir]\n", progname);
   fprintf(stderr, "  %s relate [-list file] [-force] [-relax] [-ni] name [prefix]\n", progname);
@@ -206,6 +207,8 @@ int runregister(int argc, char **argv){
     return 1;
   }
   free(tmp);
+  typemap = cbmapopenex(31);
+  cbglobalgc(typemap, (void (*)(void *))cbmapclose);
   name = NULL;
   dir = NULL;
   lfile = NULL;
@@ -246,6 +249,9 @@ int runregister(int argc, char **argv){
         xtype = argv[i];
         if(++i >= argc) usage();
         xcmd = argv[i];
+      } else if(!strcmp(argv[i], "-xtype")){
+        if(++i >= argc || ++i >= argc) usage();
+        cbmapput(typemap, argv[i-1], -1, argv[i], -1, FALSE);
       } else if(!strcmp(argv[i], "-xt")){
         xtext = TRUE;
       } else if(!strcmp(argv[i], "-xm")){
@@ -861,7 +867,7 @@ int indexdir(ODEUM *odeum, VILLA *mtdb, const char *name, const char *dir){
 int indexfile(ODEUM *odeum, VILLA *mtdb, const char *name, const char *file, const char *ruri){
   static int cnt = 0;
   char *buf, *uri;
-  const char *date, *title;
+  const char *cmd, *date, *title;
   int size, mtime, wnum, bnum, err;
   ODDOC *doc;
   if(!cbfilestat(file, NULL, &size, &mtime)){
@@ -870,8 +876,32 @@ int indexfile(ODEUM *odeum, VILLA *mtdb, const char *name, const char *file, con
   }
   date = datestr(mtime);
   doc = NULL;
+  if(cbmaprnum(typemap) > 0){
+    xtype = ruri;
+    xcmd = "";
+    ruri = NULL;
+  }
   if(fwmatchlist(file, iprelist) || (isiz > 0 && size > isiz)){
     return TRUE;
+  } else if(xtype && xtype[0] != '\0' && (cmd = cbmapget(typemap, xtype, -1, NULL))){
+    xcmd = cmd;
+    uri = filetouri(file);
+    if(!force && !ishot(mtdb, file, mtime)){
+      printfinfo("%s: passed", uri);
+      free(uri);
+      relaxsystem(RLXPASSINC);
+      return TRUE;
+    }
+    if(!(buf = cbreadfile(file, NULL))){
+      printfinfo("%s: file cannot be opened", uri);
+      free(uri);
+      return TRUE;
+    }
+    doc = docext(uri, file, buf, size);
+    if(!oddocgetattr(doc, "date")) oddocaddattr(doc, "date", date);
+    free(uri);
+    free(buf);
+    ruri = NULL;
   } else if(bwimatchlist(file, tsuflist)){
     uri = filetouri(file);
     if(!force && !ishot(mtdb, file, mtime)){
