@@ -27,7 +27,7 @@
 #define CB_MAPCSUNIT   64                /* small allocation unit size of map concatenation */
 #define CB_MAPCBUNIT   256               /* big allocation unit size of map concatenation */
 #define CB_MSGBUFSIZ   256               /* size of a buffer for log message */
-#define CB_IOBUFSIZ    4096              /* size of an I/O buffer */
+#define CB_IOBUFSIZ    8192              /* size of an I/O buffer */
 #define CB_FILEMODE    00644             /* permission of a creating file */
 #define CB_ENCBUFSIZ   32                /* size of a buffer for encoding name */
 #define CB_DATEBUFSIZ  64                /* size of a buffer for date expression */
@@ -35,7 +35,6 @@
 
 
 /* private function prototypes */
-static void cbmyfatal(const char *message);
 static void cbggchandler(void);
 static void cbggckeeper(void *ptr, void (*func)(void *));
 static void cbqsortsub(char *bp, int nmemb, int size, char *pswap, char *vswap,
@@ -62,13 +61,7 @@ void (*cbfatalfunc)(const char *message) = NULL;
 void *cbmalloc(size_t size){
   char *p;
   assert(size > 0 && size < INT_MAX);
-  if(!(p = malloc(size))){
-    if(cbfatalfunc){
-      cbfatalfunc("out of memory");
-    } else {
-      cbmyfatal("out of memory");
-    }
-  }
+  if(!(p = malloc(size))) cbmyfatal("out of memory");
   return p;
 }
 
@@ -77,13 +70,7 @@ void *cbmalloc(size_t size){
 void *cbrealloc(void *ptr, size_t size){
   char *p;
   assert(size > 0);
-  if(!(p = realloc(ptr, size))){
-    if(cbfatalfunc){
-      cbfatalfunc("out of memory");
-    } else {
-      cbmyfatal("out of memory");
-    }
-  }
+  if(!(p = realloc(ptr, size))) cbmyfatal("out of memory");
   return p;
 }
 
@@ -93,7 +80,7 @@ char *cbmemdup(const char *ptr, int size){
   char *p;
   assert(ptr);
   if(size < 0) size = strlen(ptr);
-  p = cbmalloc(size + 1);
+  CB_MALLOC(p, size + 1);
   memcpy(p, ptr, size);
   p[size] = '\0';
   return p;
@@ -119,7 +106,7 @@ void cbisort(void *base, int nmemb, int size, int(*compar)(const void *, const v
   int i, j;
   assert(base && nmemb >= 0 && size > 0 && compar);
   bp = (char *)base;
-  swap = cbmalloc(size);
+  CB_MALLOC(swap, size);
   for(i = 1; i < nmemb; i++){
     if(compar(bp + (i - 1) * size, bp + i * size) > 0){
       memcpy(swap, bp + i * size, size);
@@ -140,7 +127,7 @@ void cbssort(void *base, int nmemb, int size, int(*compar)(const void *, const v
   int step, bottom, i, j;
   assert(base && nmemb >= 0 && size > 0 && compar);
   bp = (char *)base;
-  swap = cbmalloc(size);
+  CB_MALLOC(swap, size);
   for(step = (nmemb - 1) / 3; step >= 0; step = (step - 1) / 3){
     if(step < 5) step = 1;
     for(bottom = 0; bottom < step; bottom++){
@@ -170,7 +157,7 @@ void cbhsort(void *base, int nmemb, int size, int(*compar)(const void *, const v
   nmemb--;
   bottom = nmemb / 2 +1;
   top = nmemb;
-  swap = cbmalloc(size);
+  CB_MALLOC(swap, size);
   while(bottom > 0){
     bottom--;
     mybot = bottom;
@@ -210,8 +197,8 @@ void cbhsort(void *base, int nmemb, int size, int(*compar)(const void *, const v
 void cbqsort(void *base, int nmemb, int size, int(*compar)(const void *, const void *)){
   char *pswap, *vswap;
   assert(base && nmemb >= 0 && size > 0 && compar);
-  pswap = cbmalloc(size);
-  vswap = cbmalloc(size);
+  CB_MALLOC(pswap, size);
+  CB_MALLOC(vswap, size);
   cbqsortsub(base, nmemb, size, pswap, vswap, compar);
   free(vswap);
   free(pswap);
@@ -271,7 +258,7 @@ int cbstrbwmatch(const char *str, const char *key){
   slen = strlen(str);
   klen = strlen(key);
   for(i = 1; i <= klen; i++){
-    if(str[slen-i] != key[klen-i] || i > slen) return FALSE;
+    if(i > slen || str[slen-i] != key[klen-i]) return FALSE;
   }
   return TRUE;
 }
@@ -284,16 +271,81 @@ int cbstrbwimatch(const char *str, const char *key){
   slen = strlen(str);
   klen = strlen(key);
   for(i = 1; i <= klen; i++){
+    if(i > slen) return FALSE;
     sc = str[slen-i];
     if(sc >= 'A' && sc <= 'Z') sc += 'a' - 'A';
     kc = key[klen-i];
     if(kc >= 'A' && kc <= 'Z') kc += 'a' - 'A';
-    if(sc != kc || i > slen) return FALSE;
+    if(sc != kc) return FALSE;
   }
   return TRUE;
 }
 
 
+/* Locate a substring in a string using KMP method. */
+char *cbstrstrkmp(const char *haystack, const char *needle){
+  int i, j, hlen, nlen;
+  char tbl[0x100];
+  assert(haystack && needle);
+  nlen = strlen(needle);
+  if(nlen >= 0x100) return strstr(haystack, needle);
+  tbl[0] = -1;
+  i = 0;
+  j = -1;
+  while(i < nlen){
+    while((j >= 0) && (needle[i] != needle[j])){
+      j = tbl[j];
+    }
+    i++;
+    j++;
+    tbl[i] = j;
+  }
+  hlen = strlen(haystack);
+  i = 0;
+  j = 0;
+  while(i < hlen && j < nlen){
+    while((j >= 0) && (haystack[i] != needle[j])){
+      j = tbl[j];
+    }
+    i++;
+    j++;
+  }
+  if(j == nlen) return (char *)(haystack + i - nlen);
+  return NULL;
+}
+
+
+/* Locate a substring in a string using BM method. */
+char *cbstrstrbm(const char *haystack, const char *needle){
+  const unsigned char *rp;
+  const char *ep;
+  unsigned char tbl[0x100];
+  int i, j, nlen, len, idx;
+  assert(haystack && needle);
+  nlen = strlen(needle);
+  if(nlen < 3 || nlen >= 0x100) return strstr(haystack, needle);
+  for(i = 0; i < 0x100; i++){
+    tbl[i] = nlen;
+  }
+  len = nlen;
+  rp = (const unsigned char *)needle;
+  while(len > 0){
+    tbl[*rp++] = --len;
+  }
+  nlen--;
+  ep = haystack + strlen(haystack) - nlen;
+  while(haystack < ep){
+    for(i = nlen; haystack[i] == needle[i]; i--){
+      if(i == 0) return (char *)haystack;
+    }
+    idx = ((unsigned char *)haystack)[i];
+    j = tbl[idx] - nlen + i;
+    haystack += j > 0 ? j : 2;
+  }
+  return NULL;
+}
+
+
 /* Convert the letters of a string to upper case. */
 char *cbstrtoupper(char *str){
   int i;
@@ -370,8 +422,8 @@ char *cbstrsqzspc(char *str){
 /* Get a datum handle. */
 CBDATUM *cbdatumopen(const char *ptr, int size){
   CBDATUM *datum;
-  datum = cbmalloc(sizeof(*datum));
-  datum->dptr = cbmalloc(CB_DATUMUNIT);
+  CB_MALLOC(datum, sizeof(*datum));
+  CB_MALLOC(datum->dptr, CB_DATUMUNIT);
   datum->dptr[0] = '\0';
   datum->dsize = 0;
   datum->asize = CB_DATUMUNIT;
@@ -401,7 +453,7 @@ void cbdatumcat(CBDATUM *datum, const char *ptr, int size){
   if(size < 0) size = strlen(ptr);
   if(datum->dsize + size >= datum->asize){
     datum->asize = datum->asize * 2 + size + 1;
-    datum->dptr = cbrealloc(datum->dptr, datum->asize);
+    CB_REALLOC(datum->dptr, datum->asize);
   }
   memmove(datum->dptr + datum->dsize, ptr, size);
   datum->dsize += size;
@@ -432,7 +484,7 @@ void cbdatumsetsize(CBDATUM *datum, int size){
   } else {
     if(size >= datum->asize){
       datum->asize = datum->asize * 2 + size + 1;
-      datum->dptr = cbrealloc(datum->dptr, datum->asize);
+      CB_REALLOC(datum->dptr, datum->asize);
     }
     memset(datum->dptr + datum->dsize, 0, (size - datum->dsize) + 1);
     datum->dsize = size;
@@ -454,9 +506,9 @@ char *cbdatumtomalloc(CBDATUM *datum, int *sp){
 /* Get a list handle. */
 CBLIST *cblistopen(void){
   CBLIST *list;
-  list = cbmalloc(sizeof(*list));
+  CB_MALLOC(list, sizeof(*list));
   list->anum = CB_LISTUNIT;
-  list->array = cbmalloc(sizeof(list->array[0]) * list->anum);
+  CB_MALLOC(list->array, sizeof(list->array[0]) * list->anum);
   list->start = 0;
   list->num = 0;
   return list;
@@ -516,9 +568,9 @@ void cblistpush(CBLIST *list, const char *ptr, int size){
   index = list->start + list->num;
   if(index >= list->anum){
     list->anum *= 2;
-    list->array = cbrealloc(list->array, list->anum * sizeof(list->array[0]));
+    CB_REALLOC(list->array, list->anum * sizeof(list->array[0]));
   }
-  list->array[index].dptr = cbmalloc((size < CB_DATUMUNIT ? CB_DATUMUNIT : size) + 1);
+  CB_MALLOC(list->array[index].dptr, (size < CB_DATUMUNIT ? CB_DATUMUNIT : size) + 1);
   memcpy(list->array[index].dptr, ptr, size);
   list->array[index].dptr[size] = '\0';
   list->array[index].dsize = size;
@@ -546,13 +598,13 @@ void cblistunshift(CBLIST *list, const char *ptr, int size){
   if(list->start < 1){
     if(list->start + list->num >= list->anum){
       list->anum *= 2;
-      list->array = cbrealloc(list->array, list->anum * sizeof(list->array[0]));
+      CB_REALLOC(list->array, list->anum * sizeof(list->array[0]));
     }
     list->start = list->anum - list->num;
     memmove(list->array + list->start, list->array, list->num * sizeof(list->array[0]));
   }
   index = list->start - 1;
-  list->array[index].dptr = cbmalloc((size < CB_DATUMUNIT ? CB_DATUMUNIT : size) + 1);
+  CB_MALLOC(list->array[index].dptr, (size < CB_DATUMUNIT ? CB_DATUMUNIT : size) + 1);
   memcpy(list->array[index].dptr, ptr, size);
   list->array[index].dptr[size] = '\0';
   list->array[index].dsize = size;
@@ -582,7 +634,7 @@ void cblistinsert(CBLIST *list, int index, const char *ptr, int size){
   index += list->start;
   if(list->start + list->num >= list->anum){
     list->anum *= 2;
-    list->array = cbrealloc(list->array, list->anum * sizeof(list->array[0]));
+    CB_REALLOC(list->array, list->anum * sizeof(list->array[0]));
   }
   memmove(list->array + index + 1, list->array + index,
           sizeof(list->array[0]) * (list->start + list->num - index));
@@ -614,7 +666,7 @@ void cblistover(CBLIST *list, int index, const char *ptr, int size){
   if(size < 0) size = strlen(ptr);
   index += list->start;
   if(size > list->array[index].dsize)
-    list->array[index].dptr = cbrealloc(list->array[index].dptr, size + 1);
+    CB_REALLOC(list->array[index].dptr, size + 1);
   memcpy(list->array[index].dptr, ptr, size);
   list->array[index].dsize = size;
   list->array[index].dptr[size] = '\0';
@@ -663,13 +715,13 @@ char *cblistdump(const CBLIST *list, int *sp){
   assert(list && sp);
   ln = cblistnum(list);
   vnumsiz = cbsetvnumbuf(vnumbuf, ln);
-  buf = cbmalloc(vnumsiz + 1);
+  CB_MALLOC(buf, vnumsiz + 1);
   memcpy(buf, vnumbuf, vnumsiz);
   bsiz = vnumsiz;
   for(i = 0; i < ln; i++){
     vbuf = cblistval(list, i, &vsiz);
     vnumsiz = cbsetvnumbuf(vnumbuf, vsiz);
-    buf = cbrealloc(buf, bsiz + vnumsiz + vsiz + 1);
+    CB_REALLOC(buf, bsiz + vnumsiz + vsiz + 1);
     memcpy(buf + bsiz, vnumbuf, vnumsiz);
     bsiz += vnumsiz;
     memcpy(buf + bsiz, vbuf, vsiz);
@@ -784,7 +836,7 @@ int cbmapput(CBMAP *map, const char *kbuf, int ksiz, const char *vbuf, int vsiz,
       }
     }
   }
-  datum = cbmalloc(sizeof(*datum));
+  CB_MALLOC(datum, sizeof(*datum));
   datum->kbuf = cbmemdup(kbuf, ksiz);
   datum->ksiz = ksiz;
   datum->vbuf = cbmemdup(vbuf, vsiz);
@@ -833,7 +885,7 @@ void cbmapputcat(CBMAP *map, const char *kbuf, int ksiz, const char *vbuf, int v
         asiz = datum->vsiz + vsiz;
         unit = asiz <= CB_MAPCSUNIT ? CB_MAPCSUNIT : CB_MAPCBUNIT;
         asiz = (asiz - 1) + unit - (asiz - 1) % unit;
-        datum->vbuf = cbrealloc(datum->vbuf, asiz + 1);
+        CB_REALLOC(datum->vbuf, asiz + 1);
         memcpy(datum->vbuf + datum->vsiz, vbuf, vsiz);
         *(datum->vbuf + datum->vsiz + vsiz) = '\0';
         datum->vsiz += vsiz;
@@ -843,10 +895,10 @@ void cbmapputcat(CBMAP *map, const char *kbuf, int ksiz, const char *vbuf, int v
   }
   unit = vsiz <= CB_MAPCSUNIT ? CB_MAPCSUNIT : CB_MAPCBUNIT;
   asiz = (vsiz - 1) + unit - (vsiz - 1) % unit;
-  datum = cbmalloc(sizeof(*datum));
+  CB_MALLOC(datum, sizeof(*datum));
   datum->kbuf = cbmemdup(kbuf, ksiz);
   datum->ksiz = ksiz;
-  datum->vbuf = cbmalloc(asiz + 1);
+  CB_MALLOC(datum->vbuf, asiz + 1);
   memcpy(datum->vbuf, vbuf, vsiz);
   *(datum->vbuf + vsiz) = '\0';
   datum->vsiz = vsiz;
@@ -1090,20 +1142,20 @@ char *cbmapdump(CBMAP *map, int *sp){
   assert(map && sp);
   rn = cbmaprnum(map);
   vnumsiz = cbsetvnumbuf(vnumbuf, rn);
-  buf = cbmalloc(vnumsiz + 1);
+  CB_MALLOC(buf, vnumsiz + 1);
   memcpy(buf, vnumbuf, vnumsiz);
   bsiz = vnumsiz;
   cbmapiterinit(map);
   while((kbuf = cbmapiternext(map, &ksiz)) != NULL){
     vbuf = cbmapget(map, kbuf, ksiz, &vsiz);
     vnumsiz = cbsetvnumbuf(vnumbuf, ksiz);
-    buf = cbrealloc(buf, bsiz + vnumsiz + ksiz + 1);
+    CB_REALLOC(buf, bsiz + vnumsiz + ksiz + 1);
     memcpy(buf + bsiz, vnumbuf, vnumsiz);
     bsiz += vnumsiz;
     memcpy(buf + bsiz, kbuf, ksiz);
     bsiz += ksiz;
     vnumsiz = cbsetvnumbuf(vnumbuf, vsiz);
-    buf = cbrealloc(buf, bsiz + vnumsiz + vsiz + 1);
+    CB_REALLOC(buf, bsiz + vnumsiz + vsiz + 1);
     memcpy(buf + bsiz, vnumbuf, vnumsiz);
     bsiz += vnumsiz;
     memcpy(buf + bsiz, vbuf, vsiz);
@@ -1147,6 +1199,41 @@ CBMAP *cbmapload(const char *ptr, int size){
 }
 
 
+/* Redintegrate a serialized map and get one of the records. */
+char *cbmaploadone(const char *ptr, int size, const char *kbuf, int ksiz, int *sp){
+  const char *rp, *tkbuf, *vbuf;
+  int i, step, rn, tksiz, vsiz;
+  assert(ptr && size >= 0 && kbuf);
+  if(ksiz < 0) ksiz = strlen(kbuf);
+  rp = ptr;
+  rn = cbreadvnumbuf(rp, size, &step);
+  rp += step;
+  size -= step;
+  if(rn > size) return NULL;
+  for(i = 0; i < rn; i++){
+    if(size < 1) break;
+    tksiz = cbreadvnumbuf(rp, size, &step);
+    rp += step;
+    size -= step;
+    if(tksiz > size) break;
+    tkbuf = rp;
+    rp += tksiz;
+    if(size < 1) break;
+    vsiz = cbreadvnumbuf(rp, size, &step);
+    rp += step;
+    size -= step;
+    if(vsiz > size) break;
+    vbuf = rp;
+    rp += vsiz;
+    if(tksiz == ksiz && !memcmp(tkbuf, kbuf, ksiz)){
+      if(sp) *sp = vsiz;
+      return cbmemdup(vbuf, vsiz);
+    }
+  }
+  return NULL;
+}
+
+
 /* Allocate a formatted string on memory. */
 char *cbsprintf(const char *format, ...){
   va_list ap;
@@ -1156,7 +1243,7 @@ char *cbsprintf(const char *format, ...){
   double dnum;
   va_start(ap, format);
   assert(format);
-  buf = cbmalloc(1);
+  CB_MALLOC(buf, 1);
   len = 0;
   while(*format != '\0'){
     if(*format == '%'){
@@ -1177,35 +1264,35 @@ char *cbsprintf(const char *format, ...){
       switch(*format){
       case 'd':
         num = va_arg(ap, int);
-        buf = cbrealloc(buf, len + CB_SPMAXWIDTH + 2);
+        CB_REALLOC(buf, len + CB_SPMAXWIDTH + 2);
         len += sprintf(buf + len, cbuf, num);
         break;
       case 'o': case 'u': case 'x': case 'X': case 'c':
         unum = va_arg(ap, unsigned int);
-        buf = cbrealloc(buf, len + CB_SPMAXWIDTH + 2);
+        CB_REALLOC(buf, len + CB_SPMAXWIDTH + 2);
         len += sprintf(buf + len, cbuf, unum);
         break;
       case 'e': case 'E': case 'f': case 'g': case 'G':
         dnum = va_arg(ap, double);
-        buf = cbrealloc(buf, len + CB_SPMAXWIDTH + 2);
+        CB_REALLOC(buf, len + CB_SPMAXWIDTH + 2);
         len += sprintf(buf + len, cbuf, dnum);
         break;
       case 's':
         str = va_arg(ap, char *);
         slen = strlen(str);
-        buf = cbrealloc(buf, len + slen + 2);
+        CB_REALLOC(buf, len + slen + 2);
         memcpy(buf + len, str, slen);
         len += slen;
         break;
       case '%':
-        buf = cbrealloc(buf, len + 2);
+        CB_REALLOC(buf, len + 2);
         buf[len++] = '%';
         break;
       default:
         break;
       }
     } else {
-      buf = cbrealloc(buf, len + 2);
+      CB_REALLOC(buf, len + 2);
       buf[len++] = *format;
     }
     format++;
@@ -1223,7 +1310,7 @@ char *cbreplace(const char *str, CBMAP *pairs){
   const char *key, *val;
   assert(str && pairs);
   bsiz = CB_DATUMUNIT;
-  buf = cbmalloc(bsiz);
+  CB_MALLOC(buf, bsiz);
   wi = 0;
   while(*str != '\0'){
     rep = FALSE;
@@ -1236,7 +1323,7 @@ char *cbreplace(const char *str, CBMAP *pairs){
         val = cbmapget(pairs, key, ksiz, &vsiz);
         if(wi + vsiz >= bsiz){
           bsiz = bsiz * 2 + vsiz;
-          buf = cbrealloc(buf, bsiz);
+          CB_REALLOC(buf, bsiz);
         }
         memcpy(buf + wi, val, vsiz);
         wi += vsiz;
@@ -1248,13 +1335,13 @@ char *cbreplace(const char *str, CBMAP *pairs){
     if(!rep){
       if(wi + 1 >= bsiz){
         bsiz = bsiz * 2 + 1;
-        buf = cbrealloc(buf, bsiz);
+        CB_REALLOC(buf, bsiz);
       }
       buf[wi++] = *str;
       str++;
     }
   }
-  buf = cbrealloc(buf, wi + 1);
+  CB_REALLOC(buf, wi + 1);
   buf[wi] = '\0';
   return buf;
 }
@@ -1294,17 +1381,23 @@ CBLIST *cbsplit(const char *ptr, int size, const char *delim){
 
 /* Read whole data of a file. */
 char *cbreadfile(const char *name, int *sp){
-  int fd, size, rv;
+  struct stat sbuf;
   char iobuf[CB_IOBUFSIZ], *buf;
+  int fd, size, asiz, rv;
+  asiz = CB_IOBUFSIZ * 2;
   if(name){
     if((fd = open(name, O_RDONLY, 0)) == -1) return NULL;
+    if(fstat(fd, &sbuf) != -1) asiz = sbuf.st_size + 1;
   } else {
     fd = 0;
   }
-  buf = cbmalloc(1);
+  CB_MALLOC(buf, asiz + 1);
   size = 0;
   while((rv = read(fd, iobuf, CB_IOBUFSIZ)) > 0){
-    buf = cbrealloc(buf, size + rv + 1);
+    if(size + rv >= asiz){
+      asiz = asiz * 2 + size;
+      CB_REALLOC(buf, asiz + 1);
+    }
     memcpy(buf + size, iobuf, rv);
     size += rv;
   }
@@ -1392,7 +1485,7 @@ CBLIST *cbdirlist(const char *name){
 int cbfilestat(const char *name, int *isdirp, int *sizep, int *mtimep){
   struct stat sbuf;
   assert(name);
-  if(stat(name, &sbuf) == -1) return FALSE;
+  if(lstat(name, &sbuf) == -1) return FALSE;
   if(isdirp) *isdirp = S_ISDIR(sbuf.st_mode);
   if(sizep) *sizep = (int)sbuf.st_size;
   if(mtimep) *mtimep = (int)sbuf.st_mtime;
@@ -1486,7 +1579,7 @@ char *cburlencode(const char *ptr, int size){
   int i, c;
   assert(ptr);
   if(size < 0) size = strlen(ptr);
-  buf = cbmalloc(size * 3 + 1);
+  CB_MALLOC(buf, size * 3 + 1);
   wp = buf;
   for(i = 0; i < size; i++){
     c = ((unsigned char *)ptr)[i];
@@ -1558,21 +1651,21 @@ char *cbbaseencode(const char *ptr, int size){
   int i;
   assert(ptr);
   if(size < 0) size = strlen(ptr);
-  buf = cbmalloc(4 * (size + 2) / 3 + 1);
+  CB_MALLOC(buf, 4 * (size + 2) / 3 + 1);
   obj = (const unsigned char *)ptr;
   wp = buf;
   for(i = 0; i < size; i += 3){
     switch(size - i){
     case 1:
       *wp++ = tbl[obj[0] >> 2];
-      *wp++ = tbl[((obj[0] & 3) << 4) + (obj[1] >> 4)];
+      *wp++ = tbl[(obj[0] & 3) << 4];
       *wp++ = '=';
       *wp++ = '=';
       break;
     case 2:
       *wp++ = tbl[obj[0] >> 2];
       *wp++ = tbl[((obj[0] & 3) << 4) + (obj[1] >> 4)];
-      *wp++ = tbl[((obj[1] & 0xf) << 2) + (obj[2] >> 6)];
+      *wp++ = tbl[(obj[1] & 0xf) << 2];
       *wp++ = '=';
       break;
     default:
@@ -1598,7 +1691,7 @@ char *cbbasedecode(const char *str, int *sp){
   bpos = 0;
   eqcnt = 0;
   len = strlen(str);
-  obj = cbmalloc(len + 1);
+  CB_MALLOC(obj, len + 1);
   wp = obj;
   while(bpos < len && eqcnt == 0){
     bits = 0;
@@ -1657,7 +1750,8 @@ char *cbquoteencode(const char *ptr, int size){
   assert(ptr);
   if(size < 0) size = strlen(ptr);
   rp = (const unsigned char *)ptr;
-  buf = wp = cbmalloc(size * 3 + 1);
+  CB_MALLOC(buf, size * 3 + 1);
+  wp = buf;
   cols = 0;
   for(i = 0; i < size; i++){
     if(rp[i] == '=' || (rp[i] < 0x20 && rp[i] != '\r' && rp[i] != '\n' && rp[i] != '\t') ||
@@ -1678,7 +1772,8 @@ char *cbquoteencode(const char *ptr, int size){
 char *cbquotedecode(const char *str, int *sp){
   char *buf, *wp;
   assert(str);
-  buf = wp = cbmalloc(strlen(str) + 1);
+  CB_MALLOC(buf, strlen(str) + 1);
+  wp = buf;
   for(; *str != '\0'; str++){
     if(*str == '='){
       str++;
@@ -1743,7 +1838,7 @@ char *cbmimebreak(const char *ptr, int size, CBMAP *attrs, int *sp){
     }
   }
   if(head && attrs){
-    hbuf = cbmalloc(hlen + 1);
+    CB_MALLOC(hbuf, hlen + 1);
     wi = 0;
     for(i = 0; i < hlen; i++){
       if(head[i] == '\r') continue;
@@ -1791,10 +1886,17 @@ char *cbmimebreak(const char *ptr, int size, CBMAP *attrs, int *sp){
             cbmapput(attrs, "CHARSET", -1, pv, ep - pv, TRUE);
           } else if(cbstrfwimatch(ep, "boundary=")){
             ep += 9;
-            if(ep[0] == '"') ep++;
-            pv = ep;
-            while(ep[0] != '\0' && ep[0] != ' ' && ep[0] != '"' && ep[0] != ';'){
+            if(ep[0] == '"'){
               ep++;
+              pv = ep;
+              while(ep[0] != '\0' && ep[0] != '"'){
+                ep++;
+              }
+            } else {
+              pv = ep;
+              while(ep[0] != '\0' && ep[0] != ' ' && ep[0] != '"' && ep[0] != ';'){
+                ep++;
+              }
             }
             cbmapput(attrs, "BOUNDARY", -1, pv, ep - pv, TRUE);
           }
@@ -1851,7 +1953,7 @@ CBLIST *cbmimeparts(const char *ptr, int size, const char *boundary){
   pv = NULL;
   for(i = 0; i < size; i++){
     if(ptr[i] == '-' && ptr[i+1] == '-' && i + 2 + blen < size &&
-       cbstrfwmatch(ptr + i + 2, boundary)){
+       cbstrfwmatch(ptr + i + 2, boundary) && strchr("\t\n\v\f\r ", ptr[i+2+blen])){
       pv = ptr + i + 2 + blen;
       if(*pv == '\r') pv++;
       if(*pv == '\n') pv++;
@@ -1863,7 +1965,7 @@ CBLIST *cbmimeparts(const char *ptr, int size, const char *boundary){
   if(!pv) return list;
   for(i = 0; i < size; i++){
     if(ptr[i] == '-' && ptr[i+1] == '-' && i + 2 + blen < size &&
-       cbstrfwmatch(ptr + i + 2, boundary)){
+       cbstrfwmatch(ptr + i + 2, boundary) && strchr("\t\n\v\f\r -", ptr[i+2+blen])){
       ep = ptr + i;
       if(ep > ptr && ep[-1] == '\n') ep--;
       if(ep > ptr && ep[-1] == '\r') ep--;
@@ -1883,7 +1985,7 @@ char *cbmimeencode(const char *str, const char *encname, int base){
   int len;
   assert(str && encname);
   len = strlen(str);
-  buf = cbmalloc(len * 3 + strlen(encname) + 16);
+  CB_MALLOC(buf, len * 3 + strlen(encname) + 16);
   wp = buf;
   wp += sprintf(wp, "=?%s?%c?", encname, base ? 'B' : 'Q');
   enc = base ? cbbaseencode(str, len) : cbquoteencode(str, len);
@@ -1900,7 +2002,7 @@ char *cbmimedecode(const char *str, char *enp){
   int quoted;
   assert(str);
   if(enp) sprintf(enp, "US-ASCII");
-  buf = cbmalloc(strlen(str) + 1);
+  CB_MALLOC(buf, strlen(str) + 1);
   wp = buf;
   while(*str != '\0'){
     if(cbstrfwmatch(str, "=?")){
@@ -1999,7 +2101,7 @@ char *cbcsvescape(const char *str){
   char *buf, *wp;
   int i;
   assert(str);
-  buf = cbmalloc(strlen(str) * 2 + 3);
+  CB_MALLOC(buf, strlen(str) * 2 + 3);
   wp = buf;
   *(wp++) = '"';
   for(i = 0; str[i] != '\0'; i++){
@@ -2023,7 +2125,7 @@ char *cbcsvunescape(const char *str){
     len--;
     if(str[len-1] == '"') len--;
   }
-  buf = cbmalloc(len + 1);
+  CB_MALLOC(buf, len + 1);
   wp = buf;
   for(i = 0; i < len; i++){
     if(str[i] == '"'){
@@ -2232,7 +2334,7 @@ char *cbxmlunescape(const char *str){
 char *cbdeflate(const char *ptr, int size, int *sp){
   assert(ptr && sp);
   if(!_qdbm_deflate) return NULL;
-  return _qdbm_deflate(ptr, size, sp);
+  return _qdbm_deflate(ptr, size, sp, _QDBM_ZMZLIB);
 }
 
 
@@ -2240,7 +2342,23 @@ char *cbdeflate(const char *ptr, int size, int *sp){
 char *cbinflate(const char *ptr, int size, int *sp){
   assert(ptr && size >= 0);
   if(!_qdbm_inflate) return NULL;
-  return _qdbm_inflate(ptr, size, sp);
+  return _qdbm_inflate(ptr, size, sp, _QDBM_ZMZLIB);
+}
+
+
+/* Compress a serial object with GZIP. */
+char *cbgzencode(const char *ptr, int size, int *sp){
+  assert(ptr && sp);
+  if(!_qdbm_deflate) return NULL;
+  return _qdbm_deflate(ptr, size, sp, _QDBM_ZMGZIP);
+}
+
+
+/* Decompress a serial object compressed with GZIP. */
+char *cbgzdecode(const char *ptr, int size, int *sp){
+  assert(ptr && size >= 0);
+  if(!_qdbm_inflate) return NULL;
+  return _qdbm_inflate(ptr, size, sp, _QDBM_ZMGZIP);
 }
 
 
@@ -2553,13 +2671,8 @@ void cbproctime(double *usrp, double *sysp){
 
 /* Ensure that the standard I/O is binary mode. */
 void cbstdiobin(void){
-  if(setmode(0, O_BINARY) == -1 || setmode(1, O_BINARY) == -1 || setmode(2, O_BINARY) == -1){
-    if(cbfatalfunc){
-      cbfatalfunc("setmode failed");
-    } else {
-      cbmyfatal("setmode failed");
-    }
-  }
+  if(setmode(0, O_BINARY) == -1 || setmode(1, O_BINARY) == -1 || setmode(2, O_BINARY) == -1)
+    cbmyfatal("setmode failed");
 }
 
 
@@ -2569,13 +2682,69 @@ void cbstdiobin(void){
  *************************************************************************************************/
 
 
+/* Show error message on the standard error output and exit. */
+void *cbmyfatal(const char *message){
+  char buf[CB_MSGBUFSIZ];
+  assert(message);
+  if(cbfatalfunc){
+    cbfatalfunc(message);
+  } else {
+    sprintf(buf, "fatal error: %s\n", message);
+    write(2, buf, strlen(buf));
+  }
+  exit(1);
+  return NULL;
+}
+
+
+/* Create a datum handle from an allocated region. */
+CBDATUM *cbdatumopenbuf(char *ptr, int size){
+  CBDATUM *datum;
+  assert(ptr && size >= 0);
+  CB_REALLOC(ptr, size + 1);
+  CB_MALLOC(datum, sizeof(*datum));
+  datum->dptr = ptr;
+  datum->dptr[size] = '\0';
+  datum->dsize = size;
+  datum->asize = size;
+  return datum;
+}
+
+
+/* Set a buffer to a datum handle. */
+void cbdatumsetbuf(CBDATUM *datum, char *ptr, int size){
+  assert(datum && ptr && size >= 0);
+  free(datum->dptr);
+  CB_REALLOC(ptr, size + 1);
+  datum->dptr = ptr;
+  datum->dptr[size] = '\0';
+  datum->dsize = size;
+  datum->asize = size;
+}
+
+
+/* Add an allocated element at the end of a list. */
+void cblistpushbuf(CBLIST *list, char *ptr, int size){
+  int index;
+  assert(list && ptr && size >= 0);
+  index = list->start + list->num;
+  if(index >= list->anum){
+    list->anum *= 2;
+    CB_REALLOC(list->array, list->anum * sizeof(list->array[0]));
+  }
+  list->array[index].dptr = ptr;
+  list->array[index].dsize = size;
+  list->num++;
+}
+
+
 /* Get a map handle with specifying the number of buckets. */
 CBMAP *cbmapopenex(int bnum){
   CBMAP *map;
   int i;
   assert(bnum > 0);
-  map = cbmalloc(sizeof(*map));
-  map->buckets = cbmalloc(sizeof(map->buckets[0]) * bnum);
+  CB_MALLOC(map, sizeof(*map));
+  CB_MALLOC(map->buckets, sizeof(map->buckets[0]) * bnum);
   for(i = 0; i < bnum; i++){
     map->buckets[i] = NULL;
   }
@@ -2588,23 +2757,62 @@ CBMAP *cbmapopenex(int bnum){
 }
 
 
+/* Store a record with an allocated region. */
+void cbmapputvbuf(CBMAP *map, const char *kbuf, int ksiz, char *vbuf, int vsiz){
+  CBMAPDATUM *datum, **entp;
+  int bidx, hash, kcmp;
+  assert(map && kbuf && ksiz >= 0 && vbuf && vsiz >= 0);
+  bidx = cbfirsthash(kbuf, ksiz) % map->bnum;
+  datum = map->buckets[bidx];
+  entp = map->buckets + bidx;
+  hash = cbsecondhash(kbuf, ksiz);
+  while(datum){
+    if(hash > datum->hash){
+      entp = (CBMAPDATUM **)&(datum->left);
+      datum = (CBMAPDATUM *)datum->left;
+    } else if(hash < datum->hash){
+      entp = (CBMAPDATUM **)&(datum->right);
+      datum = (CBMAPDATUM *)datum->right;
+    } else {
+      kcmp = cbkeycmp(kbuf, ksiz, datum->kbuf, datum->ksiz);
+      if(kcmp < 0){
+        entp = (CBMAPDATUM **)&(datum->left);
+        datum = (CBMAPDATUM *)datum->left;
+      } else if(kcmp > 0){
+        entp = (CBMAPDATUM **)&(datum->right);
+        datum = (CBMAPDATUM *)datum->right;
+      } else {
+        free(datum->vbuf);
+        datum->vbuf = vbuf;
+        datum->vsiz = vsiz;
+        return;
+      }
+    }
+  }
+  CB_MALLOC(datum, sizeof(*datum));
+  datum->kbuf = cbmemdup(kbuf, ksiz);
+  datum->ksiz = ksiz;
+  datum->vbuf = vbuf;
+  datum->vsiz = vsiz;
+  datum->hash = hash;
+  datum->left = NULL;
+  datum->right = NULL;
+  datum->prev = (char *)map->last;
+  datum->next = NULL;
+  *entp = datum;
+  if(!map->first) map->first = datum;
+  if(map->last) map->last->next = (char *)datum;
+  map->last = datum;
+  map->rnum++;
+}
+
+
 
 /*************************************************************************************************
  * private objects
  *************************************************************************************************/
 
 
-/* Show error message on the standard error output and exit.
-   `message' specifies an error message. */
-static void cbmyfatal(const char *message){
-  char buf[CB_MSGBUFSIZ];
-  assert(message);
-  sprintf(buf, "fatal error: %s\n", message);
-  write(2, buf, strlen(buf));
-  exit(1);
-}
-
-
 /* Handler to invoke the global garbage collector. */
 static void cbggchandler(void){
   cbggckeeper(NULL, NULL);
@@ -2635,20 +2843,14 @@ static void cbggckeeper(void *ptr, void (*func)(void *)){
     return;
   }
   if(!parray){
-    parray = cbmalloc(sizeof(void *) * asiz);
-    farray = cbmalloc(sizeof(void *) * asiz);
-    if(atexit(cbggchandler) != 0){
-      if(cbfatalfunc){
-        cbfatalfunc("gc failed");
-      } else {
-        cbmyfatal("gc failed");
-      }
-    }
+    CB_MALLOC(parray, sizeof(void *) * asiz);
+    CB_MALLOC(farray, sizeof(void *) * asiz);
+    if(atexit(cbggchandler) != 0) cbmyfatal("gc failed");
   }
   if(onum >= asiz){
     asiz *= 2;
-    parray = cbrealloc(parray, sizeof(void *) * asiz);
-    farray = cbrealloc(farray, sizeof(void *) * asiz);
+    CB_REALLOC(parray, sizeof(void *) * asiz);
+    CB_REALLOC(farray, sizeof(void *) * asiz);
   }
   parray[onum] = ptr;
   farray[onum] = func;
