@@ -73,6 +73,7 @@ CURIA *cropen(const char *name, int omode, int bnum, int dnum){
     if(omode & CR_OSPARSE) dpomode |= DP_OSPARSE;
   }
   if(omode & CR_ONOLCK) dpomode |= DP_ONOLCK;
+  if(omode & CR_OLCKNB) dpomode |= DP_OLCKNB;
   attr = NULL;
   lrnum = 0;
   if((omode & CR_OWRITER) && (omode & CR_OCREAT)){
@@ -139,7 +140,7 @@ CURIA *cropen(const char *name, int omode, int bnum, int dnum){
     }
     errno = 0;
   }
-  if(stat(name, &sbuf) == -1){
+  if(lstat(name, &sbuf) == -1){
     dpclose(attr);
     dpecodeset(DP_ESTAT, __FILE__, __LINE__);
     return NULL;
@@ -313,6 +314,25 @@ int crsetalign(CURIA *curia, int align){
 }
 
 
+/* Set the size of the free block pool of a database handle. */
+int crsetfbpsiz(CURIA *curia, int size){
+  int i, err;
+  assert(curia && size >= 0);
+  if(!curia->wmode){
+    dpecodeset(DP_EMODE, __FILE__, __LINE__);
+    return FALSE;
+  }
+  err = FALSE;
+  for(i = 0; i < curia->dnum; i++){
+    if(!dpsetfbpsiz(curia->depots[i], size)){
+      err = TRUE;
+      break;
+    }
+  }
+  return err ? FALSE : TRUE;
+}
+
+
 /* Synchronize contents of updating a database with the files and the devices. */
 int crsync(CURIA *curia){
   int i, err;
@@ -476,7 +496,7 @@ int crremove(const char *name){
   CURIA *curia;
   char path[CR_PATHBUFSIZ];
   assert(name);
-  if(stat(name, &sbuf) == -1){
+  if(lstat(name, &sbuf) == -1){
     dpecodeset(DP_ESTAT, __FILE__, __LINE__);
     return FALSE;
   }
@@ -511,7 +531,7 @@ int crrepair(const char *name){
   dnum = 0;
   for(i = 1; i <= CR_DPMAX; i++){
     sprintf(path, "%s%c%04d%c%s", name, MYPATHCHR, i, MYPATHCHR, CR_DPNAME);
-    if(stat(path, &sbuf) != -1){
+    if(lstat(path, &sbuf) != -1){
       dnum++;
       if(!dprepair(path)) err = TRUE;
       if((tdepot = dpopen(path, DP_OREADER, -1)) != NULL){
@@ -526,7 +546,7 @@ int crrepair(const char *name){
   if((tcuria = cropen(path, CR_OWRITER | CR_OCREAT | CR_OTRUNC, bnum, dnum)) != NULL){
     for(i = 1; i <= CR_DPMAX; i++){
       sprintf(path, "%s%c%04d%c%s", name, MYPATHCHR, i, MYPATHCHR, CR_DPNAME);
-      if(stat(path, &sbuf) != -1){
+      if(lstat(path, &sbuf) != -1){
         if((tdepot = dpopen(path, DP_OREADER, -1)) != NULL){
           if(!dpiterinit(tdepot)) err = TRUE;
           while((kbuf = dpiternext(tdepot, &ksiz)) != NULL){
@@ -543,7 +563,7 @@ int crrepair(const char *name){
       }
       for(j = 0; j <= CR_DPMAX; j++){
         sprintf(path, "%s%c%04d%c%s", name, MYPATHCHR, j, MYPATHCHR, CR_LOBDIR);
-        if(stat(path, &sbuf) != -1){
+        if(lstat(path, &sbuf) != -1){
           if(!crcplobdir(tcuria, path)) err = TRUE;
         }
       }
@@ -637,7 +657,7 @@ int crimportdb(CURIA *curia, const char *name){
   err = FALSE;
   for(i = 0; !err && i < CR_DPMAX; i++){
     sprintf(ipath, "%s%c%04d", name, MYPATHCHR, i + 1);
-    if(stat(ipath, &sbuf) == -1 || !S_ISREG(sbuf.st_mode)) break;
+    if(lstat(ipath, &sbuf) == -1 || !S_ISREG(sbuf.st_mode)) break;
     sprintf(opath, "%s%c%04d%s", curia->name, MYPATHCHR, i + 1, CR_TMPFSUF);
     if((depot = dpopen(opath, DP_OWRITER | DP_OCREAT | DP_OTRUNC, -1)) != NULL){
       if(dpimportdb(depot, ipath)){
@@ -681,7 +701,7 @@ int crputlob(CURIA *curia, const char *kbuf, int ksiz, const char *vbuf, int vsi
     free(path);
     return FALSE;
   }
-  be = stat(path, &sbuf) != -1 && S_ISREG(sbuf.st_mode);
+  be = lstat(path, &sbuf) != -1 && S_ISREG(sbuf.st_mode);
   mode = O_RDWR | O_CREAT;
   if(dmode & CR_DKEEP) mode |= O_EXCL;
   if(dmode & CR_DCAT){
@@ -722,7 +742,7 @@ int croutlob(CURIA *curia, const char *kbuf, int ksiz){
   }
   if(ksiz < 0) ksiz = strlen(kbuf);
   if(!(path = crgetlobpath(curia, kbuf, ksiz))) return FALSE;
-  be = stat(path, &sbuf) != -1 && S_ISREG(sbuf.st_mode);
+  be = lstat(path, &sbuf) != -1 && S_ISREG(sbuf.st_mode);
   err = FALSE;
   if(unlink(path) == -1){
     err = TRUE;
@@ -806,7 +826,7 @@ int crvsizlob(CURIA *curia, const char *kbuf, int ksiz){
   assert(curia && kbuf);
   if(ksiz < 0) ksiz = strlen(kbuf);
   if(!(path = crgetlobpath(curia, kbuf, ksiz))) return -1;
-  if(stat(path, &sbuf) == -1){
+  if(lstat(path, &sbuf) == -1){
     free(path);
     dpecodeset(DP_ENOITEM, __FILE__, __LINE__);
     return -1;
@@ -1007,7 +1027,7 @@ static int crcplobdir(CURIA *curia, const char *path){
   struct stat sbuf;
   int i, ksiz, vsiz, fd;
   assert(curia && path);
-  if(stat(path, &sbuf) == -1){
+  if(lstat(path, &sbuf) == -1){
     dpecodeset(DP_ESTAT, __FILE__, __LINE__);
     return FALSE;
   }
