@@ -49,7 +49,8 @@ enum {                                   /* enumeration for open modes */
   CR_OCREAT = 1 << 2,                    /* a writer creating */
   CR_OTRUNC = 1 << 3,                    /* a writer truncating */
   CR_ONOLCK = 1 << 4,                    /* open without locking */
-  CR_OSPARSE = 1 << 5                    /* create as sparse files */
+  CR_OLCKNB = 1 << 5,                    /* lock without blocking */
+  CR_OSPARSE = 1 << 6                    /* create as sparse files */
 };
 
 enum {                                   /* enumeration for write modes */
@@ -65,9 +66,9 @@ enum {                                   /* enumeration for write modes */
    If the mode is `CR_OWRITER', the following may be added by bitwise or: `CR_OCREAT', which
    means it creates a new database if not exist, `CR_OTRUNC', which means it creates a new
    database regardless if one exists.  Both of `CR_OREADER' and `CR_OWRITER' can be added to by
-   bitwise or: `CR_ONOLCK', which means it opens a database directory without file locking.
-   `CR_OCREAT' can be added to by bitwise or: `CR_OSPARSE', which means it creates database
-   files as sparse files.
+   bitwise or: `CR_ONOLCK', which means it opens a database directory without file locking, or
+   `CR_OLCKNB', which means locking is performed without blocking.  `CR_OCREAT' can be added to
+   by bitwise or: `CR_OSPARSE', which means it creates database files as sparse files.
    `bnum' specifies the number of elements of each bucket array.  If it is not more than 0,
    the default value is specified.  The size of each bucket array is determined on creating,
    and can not be changed except for by optimization of the database.  Suggested size of each
@@ -192,7 +193,7 @@ char *criternext(CURIA *curia, int *sp);
    `curia' specifies a database handle connected as a writer.
    `align' specifies the size of alignment.
    If successful, the return value is true, else, it is false.
-   If alignment is set to a database, the efficiency of overwriting values are improved.
+   If alignment is set to a database, the efficiency of overwriting values is improved.
    The size of alignment is suggested to be average size of the values of the records to be
    stored.  If alignment is positive, padding whose size is multiple number of the alignment
    is placed.  If alignment is negative, as `vsiz' is the size of a value, the size of padding
@@ -201,6 +202,15 @@ char *criternext(CURIA *curia, int *sp);
 int crsetalign(CURIA *curia, int align);
 
 
+/* Set the size of the free block pool of a database handle.
+   `curia' specifies a database handle connected as a writer.
+   `size' specifies the size of the free block pool of a database.
+   If successful, the return value is true, else, it is false.
+   The default size of the free block pool is 16.  If the size is greater, the space efficiency
+   of overwriting values is improved with the time efficiency sacrificed. */
+int crsetfbpsiz(CURIA *curia, int size);
+
+
 /* Synchronize updating contents with the files and the devices.
    `curia' specifies a database handle connected as a writer.
    If successful, the return value is true, else, it is false.
