@@ -45,8 +45,8 @@ typedef struct {                         /* type of structure for a database han
   int rnum;                              /* number of records */
   int fatal;                             /* whether a fatal error occured or not */
   int ioff;                              /* offset of the iterator */
-  int mroff;                             /* offset of the last moved region */
-  int mrsiz;                             /* size of the last moved region */
+  int *fbpool;                           /* free block pool */
+  int fbpsiz;                            /* size of the free block pool */
   int align;                             /* basic size of alignment */
 } DEPOT;
 
@@ -80,7 +80,8 @@ enum {                                   /* enumeration for open modes */
   DP_OCREAT = 1 << 2,                    /* a writer creating */
   DP_OTRUNC = 1 << 3,                    /* a writer truncating */
   DP_ONOLCK = 1 << 4,                    /* open without locking */
-  DP_OSPARSE = 1 << 5                    /* create as a sparse file */
+  DP_OLCKNB = 1 << 5,                    /* lock without blocking */
+  DP_OSPARSE = 1 << 6                    /* create as a sparse file */
 };
 
 enum {                                   /* enumeration for write modes */
@@ -111,9 +112,9 @@ const char *dperrmsg(int ecode);
    If the mode is `DP_OWRITER', the following may be added by bitwise or: `DP_OCREAT', which
    means it creates a new database if not exist, `DP_OTRUNC', which means it creates a new
    database regardless if one exists.  Both of `DP_OREADER' and `DP_OWRITER' can be added to by
-   bitwise or: `DP_ONOLCK', which means it opens a database file without file locking.
-   `DP_OCREAT' can be added to by bitwise or: `DP_OSPARSE', which means it creates a database
-   file as a sparse file.
+   bitwise or: `DP_ONOLCK', which means it opens a database file without file locking, or
+   `DP_OLCKNB', which means locking is performed without blocking.  `DP_OCREAT' can be added to
+   by bitwise or: `DP_OSPARSE', which means it creates a database file as a sparse file.
    `bnum' specifies the number of elements of the bucket array.  If it is not more than 0,
    the default value is specified.  The size of a bucket array is determined on creating,
    and can not be changed except for by optimization of the database.  Suggested size of a
@@ -235,7 +236,7 @@ char *dpiternext(DEPOT *depot, int *sp);
    `depot' specifies a database handle connected as a writer.
    `align' specifies the size of alignment.
    If successful, the return value is true, else, it is false.
-   If alignment is set to a database, the efficiency of overwriting values are improved.
+   If alignment is set to a database, the efficiency of overwriting values is improved.
    The size of alignment is suggested to be average size of the values of the records to be
    stored.  If alignment is positive, padding whose size is multiple number of the alignment
    is placed.  If alignment is negative, as `vsiz' is the size of a value, the size of padding
@@ -244,6 +245,15 @@ char *dpiternext(DEPOT *depot, int *sp);
 int dpsetalign(DEPOT *depot, int align);
 
 
+/* Set the size of the free block pool of a database handle.
+   `depot' specifies a database handle connected as a writer.
+   `size' specifies the size of the free block pool of a database.
+   If successful, the return value is true, else, it is false.
+   The default size of the free block pool is 16.  If the size is greater, the space efficiency
+   of overwriting values is improved with the time efficiency sacrificed. */
+int dpsetfbpsiz(DEPOT *depot, int size);
+
+
 /* Synchronize updating contents with the file and the device.
    `depot' specifies a database handle connected as a writer.
    If successful, the return value is true, else, it is false.
@@ -390,6 +400,10 @@ int dpprimenum(int num);
  *************************************************************************************************/
 
 
+#define _QDBM_VERSION  "1.8.37"
+#define _QDBM_LIBVER   1109
+
+
 /* Name of the operating system. */
 extern char *dpsysname;
 
@@ -398,6 +412,10 @@ extern char *dpsysname;
 extern int dpdbgfd;
 
 
+/* Whether this build is reentrant. */
+extern const int dpisreentrant;
+
+
 /* Set the last happened error code.
    `ecode' specifies the error code.
    `line' specifies the number of the line where the error happened. */
