@@ -28,9 +28,11 @@
 #define OD_DOCSBNUM    2039              /* initial bucket number of document database */
 #define OD_DOCSDNUM    17                /* division number of document database */
 #define OD_DOCSALIGN   -4                /* alignment of document database */
+#define OD_DOCSFBP     32                /* size of free block pool of document database */
 #define OD_INDEXBNUM   32749             /* initial bucket number of inverted index */
 #define OD_INDEXDNUM   7                 /* division number of inverted index */
 #define OD_INDEXALIGN  -2                /* alignment of inverted index */
+#define OD_INDEXFBP    32                /* size of free block pool of inverted index */
 #define OD_RDOCSLRM    81                /* records in a leaf node of reverse dictionary */
 #define OD_RDOCSNIM    192               /* records in a non-leaf node of reverse dictionary */
 #define OD_RDOCSLCN    128               /* number of leaf cache of reverse dictionary */
@@ -68,6 +70,13 @@ typedef struct {                         /* type of structure for word counting
   int num;                               /* frequency of the word */
 } ODWORD;
 
+enum {                                   /* enumeration for events binded to each character */
+  OD_EVWORD,                             /* word */
+  OD_EVSPACE,                            /* space */
+  OD_EVDELIM,                            /* delimiter */
+  OD_EVGLUE                              /* glue */
+};
+
 
 /* private global variables */
 int odindexbnum = OD_INDEXBNUM;
@@ -88,6 +97,13 @@ static int odsortcompare(const void *a, const void *b);
 static int odpurgeindex(ODEUM *odeum, const char *fname);
 static CBMAP *odpairsmap(const ODPAIR *pairs, int num);
 static int odwordcompare(const void *a, const void *b);
+static int odmatchoperator(ODEUM *odeum, CBLIST *tokens);
+static ODPAIR *odparsesubexpr(ODEUM *odeum, CBLIST *tokens, CBLIST *nwords, int *np,
+                              CBLIST *errors);
+static ODPAIR *odparseexpr(ODEUM *odeum, CBLIST *tokens, CBLIST *nwords, int *np,
+                           CBLIST *errors);
+static void odfixtokens(ODEUM *odeum, CBLIST *tokens);
+static void odcleannormalized(ODEUM *odeum, CBLIST *nwords);
 
 
 
@@ -143,22 +159,22 @@ int odput(ODEUM *odeum, ODDOC *doc, int wmax, int over){
   CBLIST *tlist;
   assert(odeum);
   if(odeum->fatal){
-    dpecode = DP_EFATAL;
+    dpecodeset(DP_EFATAL, __FILE__, __LINE__);
     return FALSE;
   }
   if(!odeum->wmode){
-    dpecode = DP_EMODE;
+    dpecodeset(DP_EMODE, __FILE__, __LINE__);
     return FALSE;
   }
   if((tmp = vlget(odeum->rdocsdb, doc->uri, -1, &tsiz)) != NULL){
     if(!over){
       free(tmp);
-      dpecode = DP_EKEEP;
+      dpecodeset(DP_EKEEP, __FILE__, __LINE__);
       return FALSE;
     }
     if(tsiz != sizeof(int) || !odoutbyid(odeum, *(int *)tmp)){
       free(tmp);
-      dpecode = DP_EBROKEN;
+      dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
       odeum->fatal = TRUE;
       return FALSE;
     }
@@ -179,8 +195,7 @@ int odput(ODEUM *odeum, ODDOC *doc, int wmax, int over){
     cblistpush(tlist, ctmp, wsiz);
   }
   tmp = cblistdump(tlist, &tsiz);
-  cbmapput(map, OD_NWORDSEXPR, sizeof(OD_NWORDSEXPR), tmp, tsiz, TRUE);
-  free(tmp);
+  cbmapputvbuf(map, OD_NWORDSEXPR, sizeof(OD_NWORDSEXPR), tmp, tsiz);
   cblistclose(tlist);
   tlist = cblistopen();
   for(i = 0; i < wmax; i++){
@@ -192,15 +207,14 @@ int odput(ODEUM *odeum, ODDOC *doc, int wmax, int over){
     }
   }
   tmp = cblistdump(tlist, &tsiz);
-  cbmapput(map, OD_AWORDSEXPR, sizeof(OD_AWORDSEXPR), tmp, tsiz, TRUE);
-  free(tmp);
+  cbmapputvbuf(map, OD_AWORDSEXPR, sizeof(OD_AWORDSEXPR), tmp, tsiz);
   cblistclose(tlist);
   tmp = cbmapdump(map, &tsiz);
   cbmapclose(map);
   if(_qdbm_deflate){
-    if(!(zbuf = _qdbm_deflate(tmp, tsiz, &zsiz))){
+    if(!(zbuf = _qdbm_deflate(tmp, tsiz, &zsiz, _QDBM_ZMRAW))){
       free(tmp);
-      dpecode = DP_EMISC;
+      dpecodeset(DP_EMISC, __FILE__, __LINE__);
       odeum->fatal = TRUE;
       return FALSE;
     }
@@ -210,7 +224,7 @@ int odput(ODEUM *odeum, ODDOC *doc, int wmax, int over){
   }
   if(!crput(odeum->docsdb, (char *)&docid, sizeof(int), tmp, tsiz, CR_DKEEP)){
     free(tmp);
-    if(dpecode == DP_EKEEP) dpecode = DP_EBROKEN;
+    if(dpecode == DP_EKEEP) dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
     odeum->fatal = TRUE;
     return FALSE;
   }
@@ -266,11 +280,11 @@ int odout(ODEUM *odeum, const char *uri){
   int tsiz, docid;
   assert(odeum && uri);
   if(odeum->fatal){
-    dpecode = DP_EFATAL;
+    dpecodeset(DP_EFATAL, __FILE__, __LINE__);
     return FALSE;
   }
   if(!odeum->wmode){
-    dpecode = DP_EMODE;
+    dpecodeset(DP_EMODE, __FILE__, __LINE__);
     return FALSE;
   }
   if(!(tmp = vlget(odeum->rdocsdb, uri, -1, &tsiz))){
@@ -279,7 +293,7 @@ int odout(ODEUM *odeum, const char *uri){
   }
   if(tsiz != sizeof(int)){
     free(tmp);
-    dpecode = DP_EBROKEN;
+    dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
     odeum->fatal = TRUE;
     return FALSE;
   }
@@ -297,11 +311,11 @@ int odoutbyid(ODEUM *odeum, int id){
   CBMAP *map;
   assert(odeum && id > 0);
   if(odeum->fatal){
-    dpecode = DP_EFATAL;
+    dpecodeset(DP_EFATAL, __FILE__, __LINE__);
     return FALSE;
   }
   if(!odeum->wmode){
-    dpecode = DP_EMODE;
+    dpecodeset(DP_EMODE, __FILE__, __LINE__);
     return FALSE;
   }
   if(!(tmp = crget(odeum->docsdb, (char *)&id, sizeof(int), 0, -1, &tsiz))){
@@ -309,9 +323,9 @@ int odoutbyid(ODEUM *odeum, int id){
     return FALSE;
   }
   if(_qdbm_inflate){
-    if(!(zbuf = _qdbm_inflate(tmp, tsiz, &zsiz))){
+    if(!(zbuf = _qdbm_inflate(tmp, tsiz, &zsiz, _QDBM_ZMRAW))){
       free(tmp);
-      dpecode = DP_EBROKEN;
+      dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
       odeum->fatal = TRUE;
       return FALSE;
     }
@@ -324,7 +338,7 @@ int odoutbyid(ODEUM *odeum, int id){
   uritmp = cbmapget(map, OD_URIEXPR, sizeof(OD_URIEXPR), &uritsiz);
   if(!uritmp || !vlout(odeum->rdocsdb, uritmp, uritsiz)){
     cbmapclose(map);
-    dpecode = DP_EBROKEN;
+    dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
     odeum->fatal = TRUE;
     return FALSE;
   }
@@ -344,7 +358,7 @@ ODDOC *odget(ODEUM *odeum, const char *uri){
   int tsiz, docid;
   assert(odeum && uri);
   if(odeum->fatal){
-    dpecode = DP_EFATAL;
+    dpecodeset(DP_EFATAL, __FILE__, __LINE__);
     return NULL;
   }
   if(!(tmp = vlget(odeum->rdocsdb, uri, -1, &tsiz))){
@@ -353,7 +367,7 @@ ODDOC *odget(ODEUM *odeum, const char *uri){
   }
   if(tsiz != sizeof(int)){
     free(tmp);
-    dpecode = DP_EBROKEN;
+    dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
     odeum->fatal = TRUE;
     return NULL;
   }
@@ -372,11 +386,11 @@ ODDOC *odgetbyid(ODEUM *odeum, int id){
   CBMAP *map;
   assert(odeum);
   if(odeum->fatal){
-    dpecode = DP_EFATAL;
+    dpecodeset(DP_EFATAL, __FILE__, __LINE__);
     return NULL;
   }
   if(id < 1){
-    dpecode = DP_ENOITEM;
+    dpecodeset(DP_ENOITEM, __FILE__, __LINE__);
     return NULL;
   }
   if(!(tmp = crget(odeum->docsdb, (char *)&id, sizeof(int), 0, -1, &tsiz))){
@@ -384,9 +398,9 @@ ODDOC *odgetbyid(ODEUM *odeum, int id){
     return NULL;
   }
   if(_qdbm_inflate){
-    if(!(zbuf = _qdbm_inflate(tmp, tsiz, &zsiz))){
+    if(!(zbuf = _qdbm_inflate(tmp, tsiz, &zsiz, _QDBM_ZMRAW))){
       free(tmp);
-      dpecode = DP_EBROKEN;
+      dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
       odeum->fatal = TRUE;
       return NULL;
     }
@@ -402,7 +416,7 @@ ODDOC *odgetbyid(ODEUM *odeum, int id){
   awordstmp = cbmapget(map, OD_AWORDSEXPR, sizeof(OD_AWORDSEXPR), &awordstsiz);
   if(!uritmp || !attrstmp || !nwordstmp || !awordstmp){
     cbmapclose(map);
-    dpecode = DP_EBROKEN;
+    dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
     odeum->fatal = TRUE;
     return NULL;
   }
@@ -430,7 +444,7 @@ int odgetidbyuri(ODEUM *odeum, const char *uri){
   int tsiz, docid;
   assert(odeum && uri);
   if(odeum->fatal){
-    dpecode = DP_EFATAL;
+    dpecodeset(DP_EFATAL, __FILE__, __LINE__);
     return -1;
   }
   if(!(tmp = vlget(odeum->rdocsdb, uri, -1, &tsiz))){
@@ -439,7 +453,7 @@ int odgetidbyuri(ODEUM *odeum, const char *uri){
   }
   if(tsiz != sizeof(int)){
     free(tmp);
-    dpecode = DP_EBROKEN;
+    dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
     odeum->fatal = TRUE;
     return -1;
   }
@@ -453,11 +467,11 @@ int odgetidbyuri(ODEUM *odeum, const char *uri){
 int odcheck(ODEUM *odeum, int id){
   assert(odeum);
   if(odeum->fatal){
-    dpecode = DP_EFATAL;
+    dpecodeset(DP_EFATAL, __FILE__, __LINE__);
     return FALSE;
   }
   if(id < 1){
-    dpecode = DP_ENOITEM;
+    dpecodeset(DP_ENOITEM, __FILE__, __LINE__);
     return FALSE;
   }
   return crvsiz(odeum->docsdb, (char *)&id, sizeof(int)) != -1;
@@ -470,7 +484,7 @@ ODPAIR *odsearch(ODEUM *odeum, const char *word, int max, int *np){
   int tsiz;
   assert(odeum && word && np);
   if(odeum->fatal){
-    dpecode = DP_EFATAL;
+    dpecodeset(DP_EFATAL, __FILE__, __LINE__);
     return NULL;
   }
   if(odeum->wmode && cbmaprnum(odeum->sortmap) > 0 &&
@@ -497,7 +511,7 @@ int odsearchdnum(ODEUM *odeum, const char *word){
   int rv;
   assert(odeum && word);
   if(odeum->fatal){
-    dpecode = DP_EFATAL;
+    dpecodeset(DP_EFATAL, __FILE__, __LINE__);
     return -1;
   }
   rv = crvsiz(odeum->indexdb, word, -1);
@@ -509,7 +523,7 @@ int odsearchdnum(ODEUM *odeum, const char *word){
 int oditerinit(ODEUM *odeum){
   assert(odeum);
   if(odeum->fatal){
-    dpecode = DP_EFATAL;
+    dpecodeset(DP_EFATAL, __FILE__, __LINE__);
     return FALSE;
   }
   return criterinit(odeum->docsdb);
@@ -523,7 +537,7 @@ ODDOC *oditernext(ODEUM *odeum){
   ODDOC *doc;
   assert(odeum);
   if(odeum->fatal){
-    dpecode = DP_EFATAL;
+    dpecodeset(DP_EFATAL, __FILE__, __LINE__);
     return NULL;
   }
   doc = NULL;
@@ -534,7 +548,7 @@ ODDOC *oditernext(ODEUM *odeum){
     }
     if(tsiz != sizeof(int)){
       free(tmp);
-      dpecode = DP_EBROKEN;
+      dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
       odeum->fatal = TRUE;
       return NULL;
     }
@@ -555,11 +569,11 @@ int odsync(ODEUM *odeum){
   char numbuf[OD_NUMBUFSIZ];
   assert(odeum);
   if(odeum->fatal){
-    dpecode = DP_EFATAL;
+    dpecodeset(DP_EFATAL, __FILE__, __LINE__);
     return FALSE;
   }
   if(!odeum->wmode){
-    dpecode = DP_EMODE;
+    dpecodeset(DP_EMODE, __FILE__, __LINE__);
     return FALSE;
   }
   if(odotcb) odotcb("odsync", odeum, "writing meta information");
@@ -604,11 +618,11 @@ int odsync(ODEUM *odeum){
 int odoptimize(ODEUM *odeum){
   assert(odeum);
   if(odeum->fatal){
-    dpecode = DP_EFATAL;
+    dpecodeset(DP_EFATAL, __FILE__, __LINE__);
     return FALSE;
   }
   if(!odeum->wmode){
-    dpecode = DP_EMODE;
+    dpecodeset(DP_EMODE, __FILE__, __LINE__);
     return FALSE;
   }
   if(!odcacheflush(odeum, "odoptimize")){
@@ -650,7 +664,7 @@ int odoptimize(ODEUM *odeum){
 char *odname(ODEUM *odeum){
   assert(odeum);
   if(odeum->fatal){
-    dpecode = DP_EFATAL;
+    dpecodeset(DP_EFATAL, __FILE__, __LINE__);
     return NULL;
   }
   return cbmemdup(odeum->name, -1);
@@ -662,7 +676,7 @@ double odfsiz(ODEUM *odeum){
   double fsiz, rv;
   assert(odeum);
   if(odeum->fatal){
-    dpecode = DP_EFATAL;
+    dpecodeset(DP_EFATAL, __FILE__, __LINE__);
     return -1;
   }
   fsiz = 0;
@@ -680,7 +694,7 @@ double odfsiz(ODEUM *odeum){
 int odbnum(ODEUM *odeum){
   assert(odeum);
   if(odeum->fatal){
-    dpecode = DP_EFATAL;
+    dpecodeset(DP_EFATAL, __FILE__, __LINE__);
     return -1;
   }
   return crbnum(odeum->indexdb);
@@ -691,7 +705,7 @@ int odbnum(ODEUM *odeum){
 int odbusenum(ODEUM *odeum){
   assert(odeum);
   if(odeum->fatal){
-    dpecode = DP_EFATAL;
+    dpecodeset(DP_EFATAL, __FILE__, __LINE__);
     return -1;
   }
   return crbusenum(odeum->indexdb);
@@ -702,7 +716,7 @@ int odbusenum(ODEUM *odeum){
 int oddnum(ODEUM *odeum){
   assert(odeum);
   if(odeum->fatal){
-    dpecode = DP_EFATAL;
+    dpecodeset(DP_EFATAL, __FILE__, __LINE__);
     return -1;
   }
   return odeum->dnum;
@@ -713,7 +727,7 @@ int oddnum(ODEUM *odeum){
 int odwnum(ODEUM *odeum){
   assert(odeum);
   if(odeum->fatal){
-    dpecode = DP_EFATAL;
+    dpecodeset(DP_EFATAL, __FILE__, __LINE__);
     return -1;
   }
   return crrnum(odeum->indexdb);
@@ -955,19 +969,19 @@ int odremove(const char *name){
   sprintf(docsname, "%s%c%s", name, MYPATHCHR, OD_DOCSNAME);
   sprintf(indexname, "%s%c%s", name, MYPATHCHR, OD_INDEXNAME);
   sprintf(rdocsname, "%s%c%s", name, MYPATHCHR, OD_RDOCSNAME);
-  if(stat(name, &sbuf) == -1){
-    dpecode = DP_ESTAT;
+  if(lstat(name, &sbuf) == -1){
+    dpecodeset(DP_ESTAT, __FILE__, __LINE__);
     return FALSE;
   }
-  if(stat(docsname, &sbuf) != -1 && !crremove(docsname)) return FALSE;
-  if(stat(indexname, &sbuf) != -1 && !crremove(indexname)) return FALSE;
-  if(stat(rdocsname, &sbuf) != -1 && !vlremove(rdocsname)) return FALSE;
+  if(lstat(docsname, &sbuf) != -1 && !crremove(docsname)) return FALSE;
+  if(lstat(indexname, &sbuf) != -1 && !crremove(indexname)) return FALSE;
+  if(lstat(rdocsname, &sbuf) != -1 && !vlremove(rdocsname)) return FALSE;
   if((list = cbdirlist(name)) != NULL){
     for(i = 0; i < cblistnum(list); i++){
       file = cblistval(list, i, NULL);
       if(!strcmp(file, MYCDIRSTR) || !strcmp(file, MYPDIRSTR)) continue;
       sprintf(path, "%s%c%s", name, MYPATHCHR, file);
-      if(stat(path, &sbuf) == -1) continue;
+      if(lstat(path, &sbuf) == -1) continue;
       if(S_ISDIR(sbuf.st_mode)){
         if(!crremove(path)) return FALSE;
       } else {
@@ -977,7 +991,7 @@ int odremove(const char *name){
     cblistclose(list);
   }
   if(rmdir(name) == -1){
-    dpecode = DP_ERMDIR;
+    dpecodeset(DP_ERMDIR, __FILE__, __LINE__);
     return FALSE;
   }
   return TRUE;
@@ -1291,6 +1305,146 @@ void odsettuning(int ibnum, int idnum, int cbnum, int csiz){
 }
 
 
+/* Break a text into words and store appearance forms and normalized form into lists. */
+void odanalyzetext(ODEUM *odeum, const char *text, CBLIST *awords, CBLIST *nwords){
+  char aword[OD_MAXWORDLEN+1], *wp;
+  int lev, wsiz;
+  assert(odeum && text && awords);
+  lev = OD_EVSPACE;
+  wsiz = 0;
+  for(; *text != '\0'; text++){
+    switch(odeum->statechars[*(unsigned char *)text]){
+    case OD_EVWORD:
+      if(wsiz > 0 && lev == OD_EVDELIM){
+        cblistpush(awords, aword, wsiz);
+        if(nwords) cblistpush(nwords, "", 0);
+        wsiz = 0;
+      }
+      if(wsiz <= OD_MAXWORDLEN){
+        aword[wsiz++] = *text;
+      }
+      lev = OD_EVWORD;
+      break;
+    case OD_EVGLUE:
+      if(wsiz > 0 && lev == OD_EVDELIM){
+        cblistpush(awords, aword, wsiz);
+        if(nwords) cblistpush(nwords, "", 0);
+        wsiz = 0;
+      }
+      if(wsiz <= OD_MAXWORDLEN){
+        aword[wsiz++] = *text;
+      }
+      lev = OD_EVGLUE;
+      break;
+    case OD_EVDELIM:
+      if(wsiz > 0 && lev != OD_EVDELIM){
+        cblistpush(awords, aword, wsiz);
+        if(nwords){
+          wp = aword;
+          aword[wsiz] = '\0';
+          while(*wp != '\0'){
+            if(*wp >= 'A' && *wp <= 'Z') *wp += 'a' - 'A';
+            wp++;
+          }
+          wp--;
+          while(wp >= aword && odeum->statechars[*(unsigned char *)wp] == OD_EVGLUE){
+            wsiz--;
+            wp--;
+          }
+          cblistpush(nwords, aword, wsiz);
+        }
+        wsiz = 0;
+      }
+      if(wsiz <= OD_MAXWORDLEN){
+        aword[wsiz++] = *text;
+      }
+      lev = OD_EVDELIM;
+      break;
+    default:
+      if(wsiz > 0){
+        cblistpush(awords, aword, wsiz);
+        if(nwords){
+          if(lev == OD_EVDELIM){
+            cblistpush(nwords, "", 0);
+          } else {
+            wp = aword;
+            aword[wsiz] = '\0';
+            while(*wp != '\0'){
+              if(*wp >= 'A' && *wp <= 'Z') *wp += 'a' - 'A';
+              wp++;
+            }
+            wp--;
+            while(wp >= aword && odeum->statechars[*(unsigned char *)wp] == OD_EVGLUE){
+              wsiz--;
+              wp--;
+            }
+            cblistpush(nwords, aword, wsiz);
+          }
+        }
+        wsiz = 0;
+      }
+      lev = OD_EVSPACE;
+      break;
+    }
+  }
+  if(wsiz > 0){
+    cblistpush(awords, aword, wsiz);
+    if(nwords){
+      if(lev == OD_EVDELIM){
+        cblistpush(nwords, "", 0);
+      } else {
+        wp = aword;
+        aword[wsiz] = '\0';
+        while(*wp != '\0'){
+          if(*wp >= 'A' && *wp <= 'Z') *wp += 'a' - 'A';
+          wp++;
+        }
+        wp--;
+        while(wp >= aword && odeum->statechars[*(unsigned char *)wp] == OD_EVGLUE){
+          wsiz--;
+          wp--;
+        }
+        cblistpush(nwords, aword, wsiz);
+      }
+    }
+    wsiz = 0;
+  }
+}
+
+
+/* Set the classes of characters used by `odanalyzetext'. */
+void odsetcharclass(ODEUM *odeum, const char *spacechars, const char *delimchars,
+                    const char *gluechars){
+  assert(odeum && spacechars && delimchars && gluechars);
+  memset(odeum->statechars, OD_EVWORD, sizeof(odeum->statechars));
+  for(; *spacechars != '\0'; spacechars++){
+    odeum->statechars[*(unsigned char *)spacechars] = OD_EVSPACE;
+  }
+  for(; *delimchars != '\0'; delimchars++){
+    odeum->statechars[*(unsigned char *)delimchars] = OD_EVDELIM;
+  }
+  for(; *gluechars != '\0'; gluechars++){
+    odeum->statechars[*(unsigned char *)gluechars] = OD_EVGLUE;
+  }
+}
+
+
+/* Query a database using a small boolean query language. */
+ODPAIR *odquery(ODEUM *odeum, const char *query, int *np, CBLIST *errors){
+  CBLIST *tokens = cblistopen();
+  CBLIST *nwords = cblistopen();
+  ODPAIR *results = NULL;
+  assert(odeum && query && np);
+  odanalyzetext(odeum, query, tokens, nwords);
+  odcleannormalized(odeum, nwords);
+  odfixtokens(odeum, tokens);
+  results = odparseexpr(odeum, tokens, nwords, np, errors);
+  cblistclose(tokens);
+  cblistclose(nwords);
+  return results;
+}
+
+
 
 /*************************************************************************************************
  * features for experts
@@ -1388,7 +1542,7 @@ static ODEUM *odopendb(const char *name, int omode, int docsbnum, int indexbnum,
   ODEUM *odeum;
   assert(name);
   if(strlen(name) > OD_NAMEMAX){
-    dpecode = DP_EMISC;
+    dpecodeset(DP_EMISC, __FILE__, __LINE__);
     return NULL;
   }
   cromode = CR_OREADER;
@@ -1409,6 +1563,10 @@ static ODEUM *odopendb(const char *name, int omode, int docsbnum, int indexbnum,
     cromode |= CR_ONOLCK;
     vlomode |= VL_ONOLCK;
   }
+  if(omode & OD_OLCKNB){
+    cromode |= CR_OLCKNB;
+    vlomode |= VL_OLCKNB;
+  }
   sprintf(docsname, "%s%c%s", name, MYPATHCHR, OD_DOCSNAME);
   sprintf(indexname, "%s%c%s", name, MYPATHCHR, OD_INDEXNAME);
   sprintf(rdocsname, "%s%c%s", name, MYPATHCHR, OD_RDOCSNAME);
@@ -1417,12 +1575,12 @@ static ODEUM *odopendb(const char *name, int omode, int docsbnum, int indexbnum,
   rdocsdb = NULL;
   if((omode & OD_OWRITER) && (omode & OD_OCREAT)){
     if(mkdir(name, OD_DIRMODE) == -1 && errno != EEXIST){
-      dpecode = DP_EMKDIR;
+      dpecodeset(DP_EMKDIR, __FILE__, __LINE__);
       return NULL;
     }
   }
-  if(stat(name, &sbuf) == -1){
-    dpecode = DP_ESTAT;
+  if(lstat(name, &sbuf) == -1){
+    dpecodeset(DP_ESTAT, __FILE__, __LINE__);
     return NULL;
   }
   inode = sbuf.st_ino;
@@ -1432,7 +1590,8 @@ static ODEUM *odopendb(const char *name, int omode, int docsbnum, int indexbnum,
     return NULL;
   }
   if(omode & OD_OWRITER){
-    if(!crsetalign(docsdb, OD_DOCSALIGN) || !crsetalign(indexdb, OD_INDEXALIGN)){
+    if(!crsetalign(docsdb, OD_DOCSALIGN) || !crsetfbpsiz(docsdb, OD_DOCSFBP) ||
+       !crsetalign(indexdb, OD_INDEXALIGN) || !crsetfbpsiz(indexdb, OD_INDEXFBP)){
       crclose(indexdb);
       crclose(docsdb);
       return NULL;
@@ -1468,7 +1627,7 @@ static ODEUM *odopendb(const char *name, int omode, int docsbnum, int indexbnum,
       vlclose(rdocsdb);
       crclose(indexdb);
       crclose(docsdb);
-      dpecode = DP_EBROKEN;
+      dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
       return NULL;
     }
   } else {
@@ -1489,6 +1648,7 @@ static ODEUM *odopendb(const char *name, int omode, int docsbnum, int indexbnum,
   odeum->dmax = dmax;
   odeum->dnum = dnum;
   odeum->ldid = -1;
+  odsetcharclass(odeum, OD_SPACECHARS, OD_DELIMCHARS, OD_GLUECHARS);
   if(odotcb) odotcb(fname, odeum, "the connection was established");
   return odeum;
 }
@@ -1578,7 +1738,7 @@ static int odcacheflushrare(ODEUM *odeum, const char *fname, double ratio){
     odotcb(fname, odeum, otmsg);
   }
   cbmapiterinit(odeum->cachemap);
-  limit = rnum * ratio;
+  limit = (int)(rnum * ratio);
   for(i = 0; i < limit && (kbuf = cbmapiternext(odeum->cachemap, &ksiz)) != NULL; i++){
     vbuf = cbmapget(odeum->cachemap, kbuf, ksiz, &vsiz);
     if(!crput(odeum->indexdb, kbuf, ksiz, vbuf, vsiz, CR_DCAT)){
@@ -1671,7 +1831,7 @@ static int odpurgeindex(ODEUM *odeum, const char *fname){
       break;
     }
     if(!(vbuf = crget(odeum->indexdb, kbuf, ksiz, 0, -1, &vsiz))){
-      dpecode = DP_EBROKEN;
+      dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
       free(kbuf);
       return FALSE;
     }
@@ -1742,5 +1902,185 @@ static int odwordcompare(const void *a, const void *b){
 }
 
 
+/* Match an operator without taking it off the token list.
+   `odeum' specifies a database handle.
+   `tokens' specifies a list handle of tokens.
+   The return value is whether the next token is an operator. */
+static int odmatchoperator(ODEUM *odeum, CBLIST *tokens){
+  const char *tk = NULL;
+  int tk_len = 0;
+  tk = cblistval(tokens, 0, &tk_len);
+  if(tk && (tk[0] == '&' || tk[0] == '|' || tk[0] == '!')) return 1;
+  return 0;
+}
+
+
+/* Implements the subexpr part of the grammar.
+   `odeum' specifies a database handle.
+   `tokens' specifies a list handle of tokens.
+   `nwords' specifies a list handle of normalized words.
+   `np' specifies the pointer to a variable to which the number of the elements of the return
+   value is assigned.
+   `errors' specifies a list handle into which error messages are stored.
+   The return value is the pointer to an array of document IDs. */
+static ODPAIR *odparsesubexpr(ODEUM *odeum, CBLIST *tokens, CBLIST *nwords, int *np,
+                              CBLIST *errors){
+  char *tk = NULL;
+  int tk_len = 0;
+  char *nword = NULL;  /* used to do the actual search, should match with tokens */
+  ODPAIR *result = NULL;
+  int result_num = 0;
+  int i;
+  double ival;
+  if((tk = cblistshift(tokens, &tk_len)) != NULL){
+    assert(tk != NULL);
+    if(tk[0] == '('){
+      free(tk);
+      /* recurse into expr */
+      result = odparseexpr(odeum, tokens, nwords, &result_num, errors);
+      /* match right token RPAREN */
+      tk = cblistshift(tokens, &tk_len);
+      /* print an error if either we didn't get anything or we didn't get a ) */
+      if(tk == NULL){
+        if(errors) cblistpush(errors, "Expression ended without closing ')'", -1);
+      } else if(tk[0] != ')'){
+        if(errors) cblistpush(errors, "Un-balanced parenthesis.", -1);
+      }
+    } else if(odeum->statechars[*(unsigned char *)tk] == 0){
+      /* Perform odsearch with the next norm word that isn't an operator. */
+      nword = cblistshift(nwords, NULL);
+      assert(nword != NULL);
+      if((result = odsearch(odeum, nword, -1, &result_num)) != NULL){
+        /* TF-IDF tuning */
+        ival = odlogarithm(result_num);
+        ival = (ival * ival) / 4.0;
+        if(ival < 4.0) ival = 4.0;
+        for(i = 0; i < result_num; i++){
+          result[i].score = (int)(result[i].score / ival);
+        }
+      }
+      free(nword);
+    } else {
+      if(errors) cblistpush(errors, "Invalid sub-expression.  Expected '(' or WORD.", -1);
+      result = cbmalloc(1);
+      result_num = 0;
+    }
+    /* done with the token */
+    free(tk);
+  }
+  *np = result_num;
+  return result;
+}
+
+
+/* Implements the actual recursive decent parser for the mini query language.
+   `odeum' specifies a database handle.
+   `tokens' specifies a list handle of tokens.
+   `nwords' specifies a list handle of normalized words.
+   `np' specifies the pointer to a variable to which the number of the elements of the return
+   value is assigned.
+   `errors' specifies a list handle into which error messages are stored.
+   The return value is the pointer to an array of document IDs.
+   It simply parses an initial subexpr, and then loops over as many (operator subexpr)
+   sequences as it can find.  The odmatchoperator function handles injecting a default &
+   between consecutive words. */
+static ODPAIR *odparseexpr(ODEUM *odeum, CBLIST *tokens, CBLIST *nwords, int *np,
+                           CBLIST *errors){
+  ODPAIR *left = NULL;
+  ODPAIR *right = NULL;
+  ODPAIR *temp = NULL;
+  int left_num = 0;
+  int right_num = 0;
+  int temp_num = 0;
+  char *op = NULL;
+  int op_len = 0;
+  if(!(left = odparsesubexpr(odeum, tokens, nwords, &left_num, errors))) return NULL;
+  /* expr ::= subexpr ( op subexpr )* */
+  while(odmatchoperator(odeum, tokens)){
+    op = cblistshift(tokens, &op_len);
+    if(!(right = odparsesubexpr(odeum, tokens, nwords, &right_num, errors))){
+      free(op);
+      free(left);
+      return NULL;
+    }
+    switch(op[0]){
+    case '&':
+      temp = odpairsand(left, left_num, right, right_num, &temp_num);
+      break;
+    case '|':
+      temp = odpairsor(left, left_num, right, right_num, &temp_num);
+      break;
+    case '!':
+      temp = odpairsnotand(left, left_num, right, right_num, &temp_num);
+      break;
+    default:
+      if(errors) cblistpush(errors, "Invalid operator.  Expected '&', '|', or '!'.", -1);
+      break;
+    }
+    if(temp){
+      /* an operator was done so we must swap it with the left */
+      free(left); left = NULL;
+      left = temp;
+      left_num = temp_num;
+    }
+    free(op);
+    if(right) free(right);
+  }
+  *np = left_num;
+  return left;
+}
+
+
+/* Processes the tokens in order to break them up further.
+   `odeum' specifies a database handle.
+   `tokens' specifies a list handle of tokens. */
+static void odfixtokens(ODEUM *odeum, CBLIST *tokens){
+  const char *tk = NULL;
+  int tk_len = 0;
+  int i = 0;
+  int lastword = 0;
+  for(i = 0; i < cblistnum(tokens); i++){
+    tk = cblistval(tokens, i, &tk_len);
+    assert(tk);
+    if(tk[0] == '&' || tk[0] == '|' || tk[0] == '!' || tk[0] == '(' || tk[0] == ')'){
+      lastword = 0;
+      if(tk_len > 1){
+        /* need to break it up for the next loop around */
+        tk = cblistremove(tokens, i, &tk_len);
+        cblistinsert(tokens, i, tk, 1);
+        cblistinsert(tokens, i+1, tk+1, tk_len-1);
+        free((char *)tk);
+      }
+    } else if(odeum->statechars[*(unsigned char *)tk] == 0){
+      /* if the last one was a word and this is a word then we need a default & between them */
+      if(lastword){
+        cblistinsert(tokens, i, "&", 1);
+        i++;
+      }
+      lastword = 1;
+    }
+  }
+}
+
+
+/* Cleans out the parts of the normalized word list that are not considered words.
+   `odeum' specifies a database handle.
+   `tokens' specifies a list handle of tokens. */
+static void odcleannormalized(ODEUM *odeum, CBLIST *nwords){
+  char *tk = NULL;
+  int tk_len = 0;
+  int i = 0;
+  for(i = 0; i < cblistnum(nwords); i++){
+    tk = (char *)cblistval(nwords, i, &tk_len);
+    if(tk_len == 0 || (!odeum->statechars[*(unsigned char *)tk] == 0)){
+      /* not a word so delete it */
+      tk = cblistremove(nwords, i, &tk_len);
+      free(tk);
+      i--;
+    }
+  }
+}
+
+
 
 /* END OF FILE */
