@@ -49,6 +49,7 @@ typedef struct {                         /* type of structure for a database han
   int dmax;                              /* max number of the document ID */
   int dnum;                              /* number of the documents */
   int ldid;                              /* ID number of the last registered document */
+  char statechars[256];                  /* state of single byte characters */
 } ODEUM;
 
 typedef struct {                         /* type of structure for a document handle */
@@ -69,7 +70,8 @@ enum {                                   /* enumeration for open modes */
   OD_OWRITER = 1 << 1,                   /* open as a writer */
   OD_OCREAT = 1 << 2,                    /* a writer creating */
   OD_OTRUNC = 1 << 3,                    /* a writer truncating */
-  OD_ONOLCK = 1 << 4                     /* open without locking */
+  OD_ONOLCK = 1 << 4,                    /* open without locking */
+  OD_OLCKNB = 1 << 5                     /* lock without blocking */
 };
 
 
@@ -79,7 +81,8 @@ enum {                                   /* enumeration for open modes */
    If the mode is `OD_OWRITER', the following may be added by bitwise or: `OD_OCREAT', which
    means it creates a new database if not exist, `OD_OTRUNC', which means it creates a new
    database regardless if one exists.  Both of `OD_OREADER' and `OD_OWRITER' can be added to by
-   bitwise or: `OD_ONOLCK', which means it opens a database directory without file locking.
+   bitwise or: `OD_ONOLCK', which means it opens a database directory without file locking, or
+   `OD_OLCKNB', which means locking is performed without blocking.
    The return value is the database handle or `NULL' if it is not successful.
    While connecting as a writer, an exclusive lock is invoked to the database directory.
    While connecting as a reader, a shared lock is invoked to the database directory.
@@ -473,6 +476,42 @@ double odvectorcosine(const int *avec, const int *bvec, int vnum);
 void odsettuning(int ibnum, int idnum, int cbnum, int csiz);
 
 
+/* Break a text into words and store appearance forms and normalized form into lists.
+   `odeum' specifies a database handle.
+   `text' specifies the string of a text.
+   `awords' specifies a list handle into which appearance form is store.
+   `nwords' specifies a list handle into which normalized form is store.  If it is `NULL', it is
+   ignored.
+   Words are separated with space characters and such delimiters as period, comma and so on. */
+void odanalyzetext(ODEUM *odeum, const char *text, CBLIST *awords, CBLIST *nwords);
+
+
+/* Set the classes of characters used by `odanalyzetext'.
+   `odeum' specifies a database handle.
+   `spacechars' spacifies a string contains space characters.
+   `delimchars' spacifies a string contains delimiter characters.
+   `gluechars' spacifies a string contains glue characters. */
+void odsetcharclass(ODEUM *odeum, const char *spacechars, const char *delimchars,
+                    const char *gluechars);
+
+
+/* Query a database using a small boolean query language.
+   `odeum' specifies a database handle.
+   'query' specifies the text of the query.
+   `np' specifies the pointer to a variable to which the number of the elements of the return
+   value is assigned.
+   `errors' specifies a list handle into which error messages are stored.  If it is `NULL', it
+   is ignored.
+   If successful, the return value is the pointer to an array, else, it is `NULL'.  Each
+   element of the array is a pair of the ID number and the score of a document, and sorted in
+   descending order of their scores.  Even if no document corresponds to the specified condition,
+   it is not error but returns an dummy array.
+   Because the region of the return value is allocated with the `malloc' call, it should be
+   released with the `free' call if it is no longer in use.  Note that each element of the array
+   of the return value can be data of a deleted document. */
+ODPAIR *odquery(ODEUM *odeum, const char *query, int *np, CBLIST *errors);
+
+
 
 /*************************************************************************************************
  * features for experts
