@@ -31,6 +31,7 @@
 #define VL_INITBNUM    32749             /* initial bucket number */
 #define VL_INITALIGN   448               /* initial size of alignment */
 #define VL_OPTALIGN    -3                /* alignment setting when optimization */
+#define VL_FBPOOLSIZ   64                /* size of free block pool */
 #define VL_PATHBUFSIZ  1024              /* size of a path buffer */
 #define VL_TMPFSUF     MYEXTSTR "vltmp"  /* suffix of a temporary file */
 #define VL_ROOTKEY     -1                /* key of the root key */
@@ -99,6 +100,7 @@ VILLA *vlopen(const char *name, int omode, VLCFUNC cmp){
     if(omode & VL_OTRUNC) dpomode |= DP_OTRUNC;
   }
   if(omode & VL_ONOLCK) dpomode |= DP_ONOLCK;
+  if(omode & VL_OLCKNB) dpomode |= DP_OLCKNB;
   if(!(depot = dpopen(name, dpomode, VL_INITBNUM))) return NULL;
   flags = dpgetflags(depot);
   zmode = 0;
@@ -124,12 +126,12 @@ VILLA *vlopen(const char *name, int omode, VLCFUNC cmp){
   if(omode & VL_OWRITER){
     flags |= VL_FLISVILLA;
     if(_qdbm_deflate && zmode) flags |= VL_FLISZLIB;
-    if(!dpsetflags(depot, flags)){
+    if(!dpsetflags(depot, flags) || !dpsetfbpsiz(depot, VL_FBPOOLSIZ)){
       dpclose(depot);
       return NULL;
     }
   }
-  villa = cbmalloc(sizeof(VILLA));
+  CB_MALLOC(villa, sizeof(VILLA));
   villa->depot = depot;
   villa->cmp = cmp;
   villa->wmode = (omode & VL_OWRITER);
@@ -336,6 +338,24 @@ char *vlget(VILLA *villa, const char *kbuf, int ksiz, int *sp){
 }
 
 
+/* Get the size of the value of a record. */
+int vlvsiz(VILLA *villa, const char *kbuf, int ksiz){
+  VLLEAF *leaf;
+  VLREC *recp;
+  int pid;
+  assert(villa && kbuf);
+  if(ksiz < 0) ksiz = strlen(kbuf);
+  if((pid = vlsearchleaf(villa, kbuf, ksiz, NULL, NULL)) == -1) return -1;
+  if(!(leaf = vlleafload(villa, pid))) return -1;
+  if(!(recp = vlrecsearch(villa, leaf, kbuf, ksiz, NULL))){
+    dpecodeset(DP_ENOITEM, __FILE__, __LINE__);
+    return -1;
+  }
+  if(!villa->tran && !vlcacheadjust(villa)) return -1;
+  return CB_DATUMSIZE(recp->first);
+}
+
+
 /* Get the number of records corresponding a key. */
 int vlvnum(VILLA *villa, const char *kbuf, int ksiz){
   VLLEAF *leaf;
@@ -440,12 +460,12 @@ char *vlgetcat(VILLA *villa, const char *kbuf, int ksiz, int *sp){
     return NULL;
   }
   rsiz = CB_DATUMSIZE(recp->first);
-  rbuf = cbmalloc(rsiz + 1);
+  CB_MALLOC(rbuf, rsiz + 1);
   memcpy(rbuf, CB_DATUMPTR(recp->first), rsiz);
   if(recp->rest){
     for(i = 0; i < CB_LISTNUM(recp->rest); i++){
       vbuf = CB_LISTVAL2(recp->rest, i, &vsiz);
-      rbuf = cbrealloc(rbuf, rsiz + vsiz + 1);
+      CB_REALLOC(rbuf, rsiz + vsiz + 1);
       memcpy(rbuf + rsiz, vbuf, vsiz);
       rsiz += vsiz;
     }
@@ -705,8 +725,8 @@ char *vlcurkey(VILLA *villa, int *sp){
 char *vlcurval(VILLA *villa, int *sp){
   VLLEAF *leaf;
   VLREC *recp;
-  const char *kbuf;
-  int ksiz;
+  const char *vbuf;
+  int vsiz;
   assert(villa);
   if(villa->curleaf == -1){
     dpecodeset(DP_ENOITEM, __FILE__, __LINE__);
@@ -718,13 +738,13 @@ char *vlcurval(VILLA *villa, int *sp){
   }
   recp = (VLREC *)CB_LISTVAL(leaf->recs, villa->curknum, NULL);
   if(villa->curvnum < 1){
-    kbuf = CB_DATUMPTR(recp->first);
-    ksiz = CB_DATUMSIZE(recp->first);
+    vbuf = CB_DATUMPTR(recp->first);
+    vsiz = CB_DATUMSIZE(recp->first);
   } else {
-    kbuf = CB_LISTVAL2(recp->rest, villa->curvnum - 1, &ksiz);
+    vbuf = CB_LISTVAL2(recp->rest, villa->curvnum - 1, &vsiz);
   }
-  if(sp) *sp = ksiz;
-  return cbmemdup(kbuf, ksiz);
+  if(sp) *sp = vsiz;
+  return cbmemdup(vbuf, vsiz);
 }
 
 
@@ -1058,7 +1078,7 @@ int vlrepair(const char *name, VLCFUNC cmp){
     if(ksiz == sizeof(int) && *(int *)kbuf < VL_NODEIDMIN && *(int *)kbuf > 0){
       if((vbuf = dpget(depot, (char *)kbuf, sizeof(int), 0, -1, &vsiz)) != NULL){
         if(_qdbm_inflate && (flags & VL_FLISZLIB) &&
-           (zbuf = _qdbm_inflate(vbuf, vsiz, &zsiz)) != NULL){
+           (zbuf = _qdbm_inflate(vbuf, vsiz, &zsiz, _QDBM_ZMRAW)) != NULL){
           free(vbuf);
           vbuf = zbuf;
           vsiz = zsiz;
@@ -1106,6 +1126,7 @@ int vlrepair(const char *name, VLCFUNC cmp){
   }
   if(!vlclose(tvilla)) err = TRUE;
   if(!dpclose(depot)) err = TRUE;
+  if(!dpremove(name)) err = TRUE;
   if(rename(path, name) == -1){
     if(!err) dpecodeset(DP_EMISC, __FILE__, __LINE__);
     err = TRUE;
@@ -1126,7 +1147,7 @@ int vlexportdb(VILLA *villa, const char *name){
   vlcurfirst(villa);
   for(i = 0; !err && (kbuf = vlcurkey(villa, &ksiz)) != NULL; i++){
     if((vbuf = vlcurval(villa, &vsiz)) != NULL){
-      nkey = cbmalloc(ksiz + VL_NUMBUFSIZ);
+      CB_MALLOC(nkey, ksiz + VL_NUMBUFSIZ);
       ki = sprintf(nkey, "%X\t", i);
       memcpy(nkey + ki, kbuf, ksiz);
       if(!dpput(depot, nkey, ki + ksiz, vbuf, vsiz, DP_DKEEP)) err = TRUE;
@@ -1188,6 +1209,81 @@ int vlimportdb(VILLA *villa, const char *name){
 
 
 
+/*************************************************************************************************
+ * features for experts
+ *************************************************************************************************/
+
+
+/* Refer to a volatile cache of a value of a record. */
+const char *vlgetcache(VILLA *villa, const char *kbuf, int ksiz, int *sp){
+  VLLEAF *leaf;
+  VLREC *recp;
+  int pid;
+  assert(villa && kbuf);
+  if(ksiz < 0) ksiz = strlen(kbuf);
+  if((pid = vlsearchleaf(villa, kbuf, ksiz, NULL, NULL)) == -1) return NULL;
+  if(!(leaf = vlleafload(villa, pid))) return NULL;
+  if(!(recp = vlrecsearch(villa, leaf, kbuf, ksiz, NULL))){
+    dpecodeset(DP_ENOITEM, __FILE__, __LINE__);
+    return NULL;
+  }
+  if(!villa->tran && !vlcacheadjust(villa)) return NULL;
+  if(sp) *sp = CB_DATUMSIZE(recp->first);
+  return CB_DATUMPTR(recp->first);
+}
+
+
+/* Refer to volatile cache of the key of the record where the cursor is. */
+const char *vlcurkeycache(VILLA *villa, int *sp){
+  VLLEAF *leaf;
+  VLREC *recp;
+  const char *kbuf;
+  int ksiz;
+  assert(villa);
+  if(villa->curleaf == -1){
+    dpecodeset(DP_ENOITEM, __FILE__, __LINE__);
+    return FALSE;
+  }
+  if(!(leaf = vlleafload(villa, villa->curleaf))){
+    villa->curleaf = -1;
+    return FALSE;
+  }
+  recp = (VLREC *)CB_LISTVAL(leaf->recs, villa->curknum, NULL);
+  kbuf = CB_DATUMPTR(recp->key);
+  ksiz = CB_DATUMSIZE(recp->key);
+  if(sp) *sp = ksiz;
+  return kbuf;
+}
+
+
+/* Refer to volatile cache of the value of the record where the cursor is. */
+const char *vlcurvalcache(VILLA *villa, int *sp){
+  VLLEAF *leaf;
+  VLREC *recp;
+  const char *vbuf;
+  int vsiz;
+  assert(villa);
+  if(villa->curleaf == -1){
+    dpecodeset(DP_ENOITEM, __FILE__, __LINE__);
+    return FALSE;
+  }
+  if(!(leaf = vlleafload(villa, villa->curleaf))){
+    villa->curleaf = -1;
+    return FALSE;
+  }
+  recp = (VLREC *)CB_LISTVAL(leaf->recs, villa->curknum, NULL);
+  if(villa->curvnum < 1){
+    vbuf = CB_DATUMPTR(recp->first);
+    vsiz = CB_DATUMSIZE(recp->first);
+  } else {
+    vbuf = CB_LISTVAL2(recp->rest, villa->curvnum - 1, &vsiz);
+  }
+  if(sp) *sp = vsiz;
+  return vbuf;
+}
+
+
+
 /*************************************************************************************************
  * private objects
  *************************************************************************************************/
@@ -1438,7 +1534,7 @@ static int vlleafsave(VILLA *villa, VLLEAF *leaf){
     }
   }
   if(_qdbm_deflate && villa->zmode){
-    if(!(zbuf = _qdbm_deflate(CB_DATUMPTR(buf), CB_DATUMSIZE(buf), &zsiz))){
+    if(!(zbuf = _qdbm_deflate(CB_DATUMPTR(buf), CB_DATUMSIZE(buf), &zsiz, _QDBM_ZMRAW))){
       cbdatumclose(buf);
       if(dpecode == DP_EMODE) dpecodeset(DP_EMISC, __FILE__, __LINE__);
       return FALSE;
@@ -1489,7 +1585,7 @@ static VLLEAF *vlleafload(VILLA *villa, int id){
     return NULL;
   }
   if(_qdbm_inflate && villa->zmode){
-    if(!(zbuf = _qdbm_inflate(buf, size, &zsiz))){
+    if(!(zbuf = _qdbm_inflate(buf, size, &zsiz, _QDBM_ZMRAW))){
       dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
       free(buf);
       return NULL;
@@ -1589,11 +1685,14 @@ static int vlleafaddrec(VILLA *villa, VLLEAF *leaf, int dmode,
     if(rv == 0){
       switch(dmode){
       case VL_DOVER:
-        cbdatumclose(recp->first);
-        recp->first = cbdatumopen(vbuf, vsiz);
+        cbdatumsetsize(recp->first, 0);
+        cbdatumcat(recp->first, vbuf, vsiz);
         break;
       case VL_DKEEP:
         return FALSE;
+      case VL_DCAT:
+        cbdatumcat(recp->first, vbuf, vsiz);
+        break;
       default:
         if(!recp->rest) recp->rest = cblistopen();
         cblistpush(recp->rest, vbuf, vsiz);
